class CaveEvent < WorldEvent
  def initialize game
    @game = game
    @title = "a damp cave"
    @combat_tick = 0
    super @game.world.stores
  end

  def becomes_outpost?
    true
  end

  def init_scenes
    leave = { :next_scene => :end }
    clear = { :next_scene => :clear }

    @scenes = {
      :start => {
        :text => [ "the mouth of the cave is wide and dark.", "can't see what's inside." ],
        :options => {
          :go_inside => { :next_scene => :go_inside },
          :leave => leave
        }
      },
      :go_inside => {
        :cost => { :torch => 1 },
        :text => [ "the earth here is split, as if bearing an ancient wound." ],
        :options => {
          :continue => {
            :next_scene => { 0.3 => :a1, 0.6 => :a2, 1.0 => :a3 }
          }
        }
      },
      :a1 => {
        :text => [ "a startled beast defends its home." ],
        :combat => true,
        :enemy => :beast,
        :damage => 1,
        :health => 5,
        :max_health => 5,
        :attack_delay => 1,
        :hit => 0.8,
        :loot => {
          :fur => { :min => 1, :max => 10, :chance => 1.0 },
          :teeth => { :min => 1, :max => 5, :chance => 0.8 },
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :b1, 1.0 => :b2 }
          },
          :run => leave
        }
      },
      :a2 => {
        :text => [ 'the cave narrows a few feet in.', "the walls are moist and moss-covered." ],
        :options => {
          :squeeze => {
            :next_scene => { 0.5 => :b2, 1.0 => :b3 }
          },
          :leave => leave
        }
      },
      :a3 => {
        :text => [ 'the remains of an old camp sits just inside the cave.', 'bedrolls, torn and blackened, lay beneath a thin layer of dust.' ],
        :loot => {
          :cured_meat => { :min => 1, :max => 5, :chance => 1 },
          :torch => { :min => 1, :max => 5, :chance => 0.5 },
          :leather => { :min => 5, :max => 15, :chance => 1.0 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :b3, 1.0 => :b4 }
          },
          :run => leave
        }
      },
      :b1 => {
        :text => [ 'the body of a wanderer lies in a small cavern.', "rot's been to work on it, and some of the pieces are missing.", "can't tell what left it here." ],
        :loot => {
          :iron_sword => { :min => 1, :max => 1, :chance => 1 },
          :cured_meat => { :min => 1, :max => 5, :chance => 0.8 },
          :torch => { :min => 1, :max => 3, :chance => 0.5 }
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :b2 }
          },
          :run => leave
        }
      },
      :b2 => {
        :text => [ 'the torch sputters and dies in the damp air.', 'the darkness is absolute.' ],
        :options => {
          :light => {
            :next_scene => { 1 => :light }
          },
          :leave => leave
        }
      },
      :b3 => {
        :text => [ "a startled beast defends its home." ],
        :combat => true,
        :enemy => :beast,
        :damage => 1,
        :health => 5,
        :max_health => 5,
        :attack_delay => 1,
        :hit => 0.8,
        :loot => {
          :fur => { :min => 1, :max => 3, :chance => 1.0 },
          :teeth => { :min => 1, :max => 2, :chance => 0.8 },
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :c2 }
          },
          :run => leave
        }
      },
      :b4 => {
        :text => [ "a cave lizard attacks." ],
        :combat => true,
        :enemy => :cave_lizard,
        :damage => 3,
        :health => 6,
        :max_health => 6,
        :attack_delay => 2,
        :hit => 0.8,
        :loot => {
          :scales => { :min => 1, :max => 3, :chance => 1.0 },
          :teeth => { :min => 1, :max => 2, :chance => 0.8 },
        },
        :options => {
          :continue => {
            :next_scene => { 1 => :c2 }
          },
          :run => leave
        }
      },
      :light => {
        :cost => { :torch => 1 },
        :text => [ "a startled beast defends its home." ],
        :combat => true,
        :enemy => :beast,
        :damage => 3,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 10,
        :max_health => 10,
        :loot => {
          :fur => { :min => 1, :max => 3, :chance => 1 },
          :teeth => { :min => 1, :max => 3, :chance => 1 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.5 => :end1, 1.0 => :end2 }
          },
          :run => leave
        }
      },
      :c2 => {
        :text => [ "a cave lizard attacks." ],
        :combat => true,
        :enemy => :lizard,
        :damage => 4,
        :hit => 0.8,
        :attack_delay => 2,
        :health => 10,
        :max_health => 10,
        :loot => {
          :scales => { :min => 1, :max => 3, :chance => 1 },
          :teeth => { :min => 1, :max => 3, :chance => 1 }
        },
        :options => {
          :continue => {
            :next_scene => { 0.7 => :end2, 1.0 => :end3 }
          },
          :run => leave
        }
      },
      :end1 => {
        :text => [ 'the nest of a large animal lies at the back of the cave.' ],
        :loot => {
          :meat => { :min => 5, :max => 10, :chance => 1 },
          :fur => { :min => 5, :max => 10, :chance => 1 },
          :scales => { :min => 5, :max => 10, :chance => 1 },
          :teeth => { :min => 5, :max => 10, :chance => 1 },
          :cloth => { :min => 1, :max => 3, :chance => 0.5 },
          :bolas => { :min => 1, :max => 3, :chance => 1.0 },
          :iron_sword => { :min => 1, :max => 1, :chance => 1 },
          :bone_spear => { :min => 1, :max => 1, :chance => 1 },
        },
        :options => { :leave => clear }
      },
      :end2 => {
        :text => [ 'a small supply cache is hidden at the back of the cave.' ],
        :loot => {
          :cloth => { :min => 1, :max => 3, :chance => 1 },
          :leather => { :min => 5, :max => 15, :chance => 1 },
          :iron => { :min => 5, :max => 10, :chance => 1 },
          :cured_meat => { :min => 5, :max => 10, :chance => 1 },
          :steel => { :min => 5, :max => 10, :chance => 1.0 },
          :bolas => { :min => 1, :max => 3, :chance => 1.0 },
          :iron_sword => { :min => 1, :max => 1, :chance => 1 },
          :bone_spear => { :min => 1, :max => 1, :chance => 1 },
        },
        :options => { :leave => clear }
      },
      :end3 => {
        :text => [ 'an old case is wedged behind a rock, covered in a thick layer of dust.' ],
        :loot => {
          :bone_spear => { :min => 1, :max => 1, :chance => 1 },
          :iron_sword => { :min => 1, :max => 1, :chance => 1 },
          :steel_sword => { :min => 1, :max => 1, :chance => 1 },
          :bolas => { :min => 1, :max => 3, :chance => 1.0 }
        },
        :options => { :leave => clear }
      }
    }
  end
end
