class OpenStructLite
  attr_accessor :hash, :__touched__

  def initialize
    @hash = {}
  end

  def to_s
    @hash.to_s
  end

  def new name
    n = OpenStructLite.new
    n.__type__ = name
    n.__created_at__ = $dragon.root.tick_count
    yield n
    n
  end

  def __elapsed_ticks_since_creation__
    $dragon.root.tick_count - @hash[:__created_at__]
  end

  def __set_hash_property__ name, args
    define_singleton_method(name) do |v|
      @__touched__ = true
      return @hash[name.to_s.gsub('=', '').to_sym] = v
    end

    return self.send(name, args[0])
  end

  def __get_hash_property__ name
    define_singleton_method(name) do
      @__touched__ = true
      result = @hash[name]
      result
    end

    return self.send(name)
  end

  def method_missing name, *args, &block
    result = nil

    if name == :[]
      result = @hash[args[0]]
    elsif name == :[]=
      if !@hash[args[0]] && args[0].is_a?(Fixnum)
        @hash[args[0]] = OpenStructLite.new
        result = @hash[args[0]] = args[1]
      else
        result = @hash[args[0]] = args[1]
      end
    elsif args.length == 0
      result = __get_hash_property__ name
    elsif args.length == 1 && name.to_s.include?('=')
      result = __set_hash_property__ name, args
    end

    return result
  end

  def inspect
    return @hash.to_s
  end

  def set! key, value
    if key.is_a? Array
      key.each { |k| @hash[k.to_sym] = value }
    else
      @hash[key.to_sym] = value
    end
  end

  def clear!
    @hash.clear
  end
end

class Dragon
  attr_accessor :root

  def initialize
    @key_released_queue = []

    @reserved_attribute_keys ||= [
      :paused,
      :sounds,
      :tick_count,
      :font_sm_sm_sm,
      :font_sm_sm,
      :font_sm,
      :font_med,
      :font_lg_lg,
      :aligned_left,
      :aligned_center,
      :aligned_right,
      :labels,
      :solids,
      :borders,
      :lines,
      :sprites,
      :mouse,
      :keyboard,
      :screen,
      :files_reloaded,
      :reloaded_files,
      :player_1_up,
      :player_1_down,
      :player_1_left,
      :player_1_right,
      :player_1_accept,
      :player_1_a,
      :player_1_b,
      :player_1_x,
      :player_1_y,
      :player_1_l,
      :player_1_r,
      :player_1_l2,
      :player_1_r2,
      :player_1_l3,
      :player_1_r3,
      :player_1_up_key_down,
      :player_1_down_key_down,
      :player_1_left_key_down,
      :player_1_right_key_down,
      :player_1_accept_key_down,
      :player_1_a_key_down,
      :player_1_b_key_down,
      :player_1_x_key_down,
      :player_1_y_key_down,
      :player_1_l_key_down,
      :player_1_r_key_down,
      :player_1_l2_key_down,
      :player_1_r2_key_down,
      :player_1_l3_key_down,
      :player_1_r3_key_down,
      :player_1_a_key_held,
      :player_1_b_key_held,
      :player_1_x_key_held,
      :player_1_y_key_held,
      :player_1_l_key_held,
      :player_1_r_key_held,
      :player_1_l2_key_held,
      :player_1_r2_key_held,
      :player_1_l3_key_held,
      :player_1_r3_key_held,
      :player_2_up,
      :player_2_down,
      :player_2_left,
      :player_2_right,
      :player_2_accept,
      :player_2_a,
      :player_2_b,
      :player_2_x,
      :player_2_y,
      :player_2_l,
      :player_2_r,
      :player_2_l2,
      :player_2_r2,
      :player_2_l3,
      :player_2_r3,
      :player_2_up_key_down,
      :player_2_down_key_down,
      :player_2_left_key_down,
      :player_2_right_key_down,
      :player_2_accept_key_down,
      :player_2_a_key_down,
      :player_2_b_key_down,
      :player_2_x_key_down,
      :player_2_y_key_down,
      :player_2_l_key_down,
      :player_2_r_key_down,
      :player_2_l2_key_down,
      :player_2_r2_key_down,
      :player_2_l3_key_down,
      :player_2_r3_key_down,
      :player_2_a_key_held,
      :player_2_b_key_held,
      :player_2_x_key_held,
      :player_2_y_key_held,
      :player_2_l_key_held,
      :player_2_r_key_held,
      :player_2_l2_key_held,
      :player_2_r2_key_held,
      :player_2_l3_key_held,
      :player_2_r3_key_held,
      :mouse_x,
      :mouse_y,
      :primitives,
      :origin_x,
      :origin_y,
      :pos_x_direction,
      :pos_y_direction
    ]
    reset
  end

  def pin_root_values
    @root.sounds        ||= []
    @root.tick_count    ||= 0
    @root.font_sm_sm_sm   = -2
    @root.font_sm_sm      = -1
    @root.font_sm         =  0
    @root.font_med        =  1
    @root.font_lg_lg      =  2
    @root.aligned_left    =  0
    @root.aligned_center  =  1
    @root.aligned_right   =  2
    @root.labels        ||= []
    @root.solids        ||= []
    @root.borders       ||= []
    @root.lines         ||= []
    @root.sprites       ||= []
    @root.primitives    ||= []
    @root.mouse         ||= OpenStructLite.new
    @root.keyboard      ||= OpenStructLite.new
    @root.keyboard.keys_released ||= OpenStructLite.new
    @root.screen ||= OpenStructLite.new
    @root.screen.w = 1280
    @root.screen.w_half = 1280 / 2
    @root.screen.h =  720
    @root.screen.h_half =  720 / 2
    @root.files_reloaded ||= []
    @root.files_reloaded = [] if !@root.files_reloaded.is_a? Array
    @root.reloaded_files ||= []
    @root.reloaded_files = [] if !@root.reloaded_files.is_a? Array
    @root.paused = false if @root.paused.nil?
    @root.origin_x ||= 0
    @root.origin_y ||= 0
    @root.pos_x_direction ||= 1
    @root.pos_y_direction ||= 1

    @root.player_1_up ||= false
    @root.player_1_down ||= false
    @root.player_1_left ||= false
    @root.player_1_right ||= false
    @root.player_1_accept ||= false
    @root.player_1_a ||= false
    @root.player_1_b ||= false
    @root.player_1_x ||= false
    @root.player_1_y ||= false
    @root.player_1_l ||= false
    @root.player_1_r ||= false
    @root.player_1_l2 ||= false
    @root.player_1_r2 ||= false
    @root.player_1_l3 ||= false
    @root.player_1_r3 ||= false
    @root.player_1_up_key_down ||= false
    @root.player_1_down_key_down ||= false
    @root.player_1_left_key_down ||= false
    @root.player_1_right_key_down ||= false
    @root.player_1_accept_key_down ||= false
    @root.player_1_a_key_down ||= false
    @root.player_1_b_key_down ||= false
    @root.player_1_x_key_down ||= false
    @root.player_1_y_key_down ||= false
    @root.player_1_l_key_down ||= false
    @root.player_1_r_key_down ||= false
    @root.player_1_l2_key_down ||= false
    @root.player_1_r2_key_down ||= false
    @root.player_1_l3_key_down ||= false
    @root.player_1_r3_key_down ||= false
    @root.player_1_a_key_held  ||= false
    @root.player_1_b_key_held  ||= false
    @root.player_1_x_key_held  ||= false
    @root.player_1_y_key_held  ||= false
    @root.player_1_l_key_held  ||= false
    @root.player_1_r_key_held  ||= false
    @root.player_1_l2_key_held ||= false
    @root.player_1_r2_key_held ||= false
    @root.player_1_l3_key_held ||= false
    @root.player_1_r3_key_held ||= false
    @root.player_2_up ||= false
    @root.player_2_down ||= false
    @root.player_2_left ||= false
    @root.player_2_right ||= false
    @root.player_2_accept ||= false
    @root.player_2_a ||= false
    @root.player_2_b ||= false
    @root.player_2_x ||= false
    @root.player_2_y ||= false
    @root.player_2_l ||= false
    @root.player_2_r ||= false
    @root.player_2_l2 ||= false
    @root.player_2_r2 ||= false
    @root.player_2_l3 ||= false
    @root.player_2_r3 ||= false
    @root.player_2_up_key_down ||= false
    @root.player_2_down_key_down ||= false
    @root.player_2_left_key_down ||= false
    @root.player_2_right_key_down ||= false
    @root.player_2_accept_key_down ||= false
    @root.player_2_a_key_down ||= false
    @root.player_2_b_key_down ||= false
    @root.player_2_x_key_down ||= false
    @root.player_2_y_key_down ||= false
    @root.player_2_l_key_down ||= false
    @root.player_2_r_key_down ||= false
    @root.player_2_l2_key_down ||= false
    @root.player_2_r2_key_down ||= false
    @root.player_2_l3_key_down ||= false
    @root.player_2_r3_key_down ||= false
    @root.player_2_a_key_held  ||= false
    @root.player_2_b_key_held  ||= false
    @root.player_2_x_key_held  ||= false
    @root.player_2_y_key_held  ||= false
    @root.player_2_l_key_held  ||= false
    @root.player_2_r_key_held  ||= false
    @root.player_2_l2_key_held ||= false
    @root.player_2_r2_key_held ||= false
    @root.player_2_l3_key_held ||= false
    @root.player_2_r3_key_held ||= false
  end

  def reset
    @root = OpenStructLite.new
    pin_root_values
    clear_input
  end

  def reserved_attributes
    @root.hash.find_all { |k, v| @reserved_attribute_keys.include? k }
  end

  def attributes
    @root.hash.find_all { |k, v| !@reserved_attribute_keys.include? k }
  end

  def to_s
    "Reserved attributes: \n" +
    reserved_attributes.map do |k, v|
      "  #{[k, v]}"
    end.join("\n") +
    "\nAttributes: \n" +
    attributes.map do |k, v|
      "  #{[k, v]}"
    end.join("\n")
  end

  def dequeue_sounds
    if @root.sounds && @root.sounds.is_a?(Array)
      @root.sounds.each do |s|
        $layout.queue_sound s
      end
      @root.sounds = []
    end
  end

  def paused?
    $dragon.root.paused
  end

  def pause!
    $dragon.root.paused = true
    puts caller
  end

  def unpause!
    clear_draw_primitives
    $dragon.root.paused = false
  end

  def calc_wrapper
    return if @root.paused

    before_tick      @root if respond_to? :before_tick
    pin_defaults if @root.reloaded_files.length > 0
    pin_root_values
    clear_draw_primitives
    @root.tick_count += 1
    @root.__touched__ = false
    tick             @root if respond_to? :tick
    after_tick       @root if respond_to? :after_tick
    dequeue_sounds
    pin_root_values
    clear_input
  rescue Exception => e
    @root.paused = true
    $layout.prepend_current_repl_code "# #{e}\n"
    $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
  end
  def clear_input
    @root.mouse_x         = false
    @root.mouse_y         = false
    @root.mouse.click     = false
    @root.keyboard.keys_released.clear!
    @root.files_reloaded.clear
    @root.reloaded_files.clear
    @root.player_1_up = false
    @root.player_1_down = false
    @root.player_1_left = false
    @root.player_1_right = false
    @root.player_1_accept = false
    @root.player_1_a = false
    @root.player_1_b = false
    @root.player_1_x = false
    @root.player_1_y = false
    @root.player_1_l = false
    @root.player_1_r = false
    @root.player_1_l2 = false
    @root.player_1_r2 = false
    @root.player_1_l3 = false
    @root.player_1_r3 = false
    @root.player_2_up = false
    @root.player_2_down = false
    @root.player_2_left = false
    @root.player_2_right = false
    @root.player_2_accept = false
    @root.player_2_a = false
    @root.player_2_b = false
    @root.player_2_x = false
    @root.player_2_y = false
    @root.player_2_l = false
    @root.player_2_r = false
    @root.player_2_l2 = false
    @root.player_2_r2 = false
    @root.player_2_l3 = false
    @root.player_2_r3 = false
  end

  def clear_draw_primitives
    @root.labels.clear
    @root.sprites.clear
    @root.lines.clear
    @root.solids.clear
    @root.borders.clear
    @root.primitives.clear
  end

  def render_wrapper

  end

  def background_color
    [255, 255, 255]
  end

  def text_color
    [0, 0, 0]
  end

  def report_labels_not_array_if_needed
    is_valid = @root.labels && @root.labels.is_a?(Array)
    failed = !is_valid
    return failed if @root.paused
    if failed
    error_string = <<-ERRORSTRING
# Error `labels` invalid:
# This is what `labels` currently looks like:
# Type:  #{@root.labels.class}
# Value: #{@root.labels}
#
# `labels` must be an array of tuples.
# For example: [
#   [  100,   100, 'hello', game.font_sm, game.aligned_left, 255,   255,  255, 255],
# ]
#
# All entries must be either an array of size 3, 8 or size 9
# which conform to the following structure:
# [
#   x         (number representing the x position),
#   y         (number representing the y position),
#   message   (value to display),
#   font_size (-2, -1, 0, 1, or 2),
#   alignment (0, 1, or 2),
#   red       (number between 0 and 255 representing the red color level),
#   green     (number between 0 and 255 representing the green color level),
#   blue      (number between 0 and 255 representing the blue color level),
#   alpha     (number between 0 and 255 representing the alpha level),
# ]
ERRORSTRING

      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    failed
  end

  def is_label_entry_valid? entry
    return false if !entry.is_a? Array
    return false if !(entry.length == 8 || entry.length == 3 || entry.length == 9)
    return false if !(entry[0].is_a?(Fixnum) || entry[0].is_a?(Float))
    return false if !(entry[1].is_a?(Fixnum) || entry[1].is_a?(Float))

    if entry.length >= 3
      return false if !(entry[0].is_a?(Fixnum) || entry[0].is_a?(Float))
      return false if !(entry[1].is_a?(Fixnum) || entry[1].is_a?(Float))
    end

    if entry.length >= 8
      return false if !(entry[3].is_a?(Fixnum) || entry[3].is_a?(Float))
      return false if !(entry[4].is_a?(Fixnum) || entry[4].is_a?(Float))
      return false if !(entry[5].is_a?(Fixnum) || entry[5].is_a?(Float))
      return false if !(entry[6].is_a?(Fixnum) || entry[6].is_a?(Float))
      return false if !(entry[7].is_a?(Fixnum) || entry[7].is_a?(Float))
      return false if !(entry[3] >= -2 && entry[3] <=   2)
      return false if !(entry[4] >=  0 && entry[4] <=   2)
      return false if !(entry[5] >=  0 && entry[5] <= 255)
      return false if !(entry[6] >=  0 && entry[6] <= 255)
      return false if !(entry[7] >=  0 && entry[7] <= 255)
    end

    if entry.length >= 9
      return false if !(entry[8].is_a?(Fixnum) || entry[8].is_a?(Float))
      return false if !(entry[8] >=  0 && entry[8] <= 255)
    end

    return true
  end

  def report_labels_item_not_correct_if_needed entry, index
    failed = !is_label_entry_valid?(entry)
    return failed if @root.paused

    if failed
      error_string = <<-ERRORSTRING
# Error `labels` item:
# The entry at index #{index} is invalid:
# Type:   #{entry.class}
# Value:  #{entry}
# #{entry.is_a?(Array) ? 'Length: ' + entry.length.to_s : ''}
# #{entry.is_a?(Array) && entry.length >= 3 ? 'Message as String: ' + entry[2].to_s : ''}
#
# All entries must be either an array of size 3, size 8, or size 9
# [
#   x         (number representing the x position),
#   y         (number representing the y position),
#   message   (value to display),
#   font_size (-2, -1, 0, 1, or 2),
#   alignment (0, 1, or 2),
#   red       (number between 0 and 255 representing the red color level),
#   green     (number between 0 and 255 representing the green color level),
#   blue      (number between 0 and 255 representing the blue color level),
#   alpha     (number between 0 and 255 representing the alpha level),
# ]
      ERRORSTRING
      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    return failed
  end

  def labels
    failed = report_labels_not_array_if_needed

    if failed
      @root.paused = true
      return []
    end

    index = 0
    (@root.labels || []).map do |l|
      failed = report_labels_item_not_correct_if_needed l, index

      if failed
        @root.paused = true
        [0, 0, ""]
      else
        index += 1
        if l.length == 3
          [
            l[0].round.to_i,
            l[1].round.to_i,
            l[2].to_s
          ]
        elsif l.length == 8
          [
            l[0].round.to_i,
            l[1].round.to_i,
            l[2].to_s,
            l[3].to_i,
            l[4].to_i,
            l[5].to_i,
            l[6].to_i,
            l[7].to_i,
          ]
        elsif l.length == 9
          [
            l[0].round.to_i,
            l[1].round.to_i,
            l[2].to_s,
            l[3].to_i,
            l[4].to_i,
            l[5].to_i,
            l[6].to_i,
            l[7].to_i,
            l[8].to_i,
          ]
        else
          [0, 0, ""]
        end
      end
    end
  end

  def report_solids_not_array_if_needed
    is_valid = @root.solids && @root.solids.is_a?(Array)
    failed = !is_valid
    return failed if @root.paused

    if failed
      error_string = <<-ERRORSTRING
# Error `solids` invalid:
# This is what `solids` currently looks like:
# Type:  #{@root.solids.class}
# Value: #{@root.solids}
#
# `solids` must be an array of tuples.
# For example: [
#   [  x,   y, width, height, red, green, blue, alpha],
#   [100, 300,    20,     80, 255,   100,   60,   255],
# ]
#
# All entries within `solids` must be an array with 4, 7 or 8 values
# that conform to the following structure:
# [
#   x      (number representing the x position),
#   y      (number representing the y position),
#   width  (number representing the width),
#   height (number representing the height),
#   red    (number between 0 and 255 representing the red color level),
#   green  (number between 0 and 255 representing the green color level),
#   blue   (number between 0 and 255 representing the blue color level),
#   alpha  (number between 0 and 255 representing the alpha level),
# ]
      ERRORSTRING
      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    return failed
  end

  def is_solid_entry_valid? entry
    return false if !entry.is_a? Array
    return false if !(entry.length == 4 || entry.length == 7 || entry.length == 8)
    return !entry.any? { |e| !(e.is_a?(Fixnum) || e.is_a?(Float)) }
  end

  def report_solids_item_not_correct_if_needed entry, index
    failed = !is_solid_entry_valid?(entry)
    return failed if @root.paused

    if failed
      error_string = <<-ERRORMESSAGE
# Error `solids` entry invalid:
# The entry at index #{index} is invalid:
# Type:   #{entry.class}
# Value:  #{entry}
# #{entry.is_a?(Array) ? 'Length: ' + entry.length.to_s : ''}
#
# All entries must be an array with 4, 7 or 8 values
# that conform to the following structure:
# [
#   x      (number representing the x position),
#   y      (number representing the y position),
#   width  (number representing the width),
#   height (number representing the height),
#   red    (number between 0 and 255 representing the red color level),
#   green  (number between 0 and 255 representing the green color level),
#   blue   (number between 0 and 255 representing the blue color level),
#   alpha  (number between 0 and 255 representing the alpha level),
# ]
ERRORMESSAGE
      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    return failed
  end

  def highlights
    failed = report_solids_not_array_if_needed

    if failed
      @root.paused = true
      return []
    end

    index = 0
    results = (@root.solids || []).map do |l|
      failed = report_solids_item_not_correct_if_needed l, index

      if failed
        @root.paused = true
        [0, 0, 0, 0, 0, 0, 0]
      else
        index += 1

        if l.length == 4
          [
            l[0].round.to_i, # x
            l[1].round.to_i, # y
            l[2].round.to_i, # w
            l[3].round.to_i, # h
            0,
            0,
            0
          ]
        elsif l.length == 7
          [
            l[0].round.to_i, # x
            l[1].round.to_i, # y
            l[2].round.to_i, # w
            l[3].round.to_i, # h
            l[4].round.to_i, # r
            l[5].round.to_i, # g
            l[6].round.to_i  # b
          ]
        elsif l.length == 8
          [
            l[0].round.to_i, # x
            l[1].round.to_i, # y
            l[2].round.to_i, # w
            l[3].round.to_i, # h
            l[4].round.to_i, # r
            l[5].round.to_i, # g
            l[6].round.to_i, # b
            l[7].round.to_i  # a
          ]
        end
      end
    end

    results.to_a
  end

  def report_borders_not_array_if_needed
    is_valid = @root.borders && @root.borders.is_a?(Array)
    failed = !is_valid
    return failed if @root.paused

    if failed
      error_string = <<-ERRORMESSAGE
# Error `borders` invalid:
# This is what `borders` currently looks like:
# Type:  #{@root.borders.class}
# Value: #{@root.borders}
#
# `borders` must be an array of tuples.
# For example: [
#   [  x,   y, width, height, red, green, blue, alpha],
#   [100, 300,    20,     80, 255,   100,   60,   255],
# ]
#
# All entries within `borders` must be an array with 4, 7 or 8 values
# that conform to the following structure:
# [
#   x      (number representing the x position),
#   y      (number representing the y position),
#   width  (number representing the width),
#   height (number representing the height),
#   red    (number between 0 and 255 representing the red color level),
#   green  (number between 0 and 255 representing the green color level),
#   blue   (number between 0 and 255 representing the blue color level),
#   alpha  (number between 0 and 255 representing the alpha level),
# ]
ERRORMESSAGE
      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    return failed
  end

  def is_border_entry_valid? entry
    return false if !entry.is_a? Array
    return false if !(entry.length == 7 || entry.length == 4 || entry.length == 8)
    return !entry.any? { |e| !(e.is_a?(Fixnum) || e.is_a?(Float)) }
  end

  def report_borders_item_not_correct_if_needed entry, index
    failed = !is_border_entry_valid?(entry)
    return failed if @root.paused

    if failed
      error_string = <<-ERRORMESSAGE
# Error `border` entry invalid:
# The entry at index #{index} is invalid:
# Type:   #{entry.class}
# Value:  #{entry}
#
# All entries within `borders` must be an array with 4, 7 or 8 values
# that conform to the following structure:
# [
#   x      (number representing the x position),
#   y      (number representing the y position),
#   width  (number representing the width),
#   height (number representing the height),
#   red    (number between 0 and 255 representing the red color level),
#   green  (number between 0 and 255 representing the green color level),
#   blue   (number between 0 and 255 representing the blue color level),
#   alpha  (number between 0 and 255 representing the alpha level),
# ]
ERRORMESSAGE
      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    return failed
  end

  def borders
    failed = report_borders_not_array_if_needed

    if failed
      @root.paused = true
      return []
    end

    index = 0

    (@root.borders || []).map do |l|
      failed = report_borders_item_not_correct_if_needed l, index

      if failed
        @root.paused = true
        [0, 0, 0, 0, 0, 0, 0]
      else
        if l.length == 4
          [
            l[0].round.to_i,  # x
            l[1].round.to_i,  # y
            l[2].round.to_i,  # w
            l[3].round.to_i   # h
          ]
        elsif l.length == 7
          [
            l[0].round.to_i, # x
            l[1].round.to_i, # y
            l[2].round.to_i, # w
            l[3].round.to_i, # h
            l[4].round.to_i, # r
            l[5].round.to_i, # g
            l[6].round.to_i, # b
          ]
        elsif l.length == 8
          [
            l[0].round.to_i, # x
            l[1].round.to_i, # y
            l[2].round.to_i, # w
            l[3].round.to_i, # h
            l[4].round.to_i, # r
            l[5].round.to_i, # g
            l[6].round.to_i, # b
            l[7].round.to_i, # a
          ]
        end
      end
    end
  end

  def report_lines_not_array_if_needed
    is_valid = @root.lines && @root.lines.is_a?(Array)
    failed = !is_valid
    return failed if @root.paused

    if failed
      error_string = <<-ERRORMESSAGE
# Error `lines` invalid:
# This is what `lines` currently looks like:
# Type:  #{@root.lines.class}
# Value: #{@root.lines}
#
# `lines` must be an array of tuples.
# For example: [
#   [  x,   y,  x1,  x2, red, green, blue, alpha],
#   [100, 300, 120, 380, 255,   100,   60,   255],
# ]
#
# All entries within `lines` must be an array with 4, 7 or 8 values
# that conform to the following structure:
# [
#   x     (number representing the x position),
#   y     (number representing the y position),
#   x1    (number representing the width),
#   x2    (number representing the height),
#   red   (number between 0 and 255 representing the red color level),
#   green (number between 0 and 255 representing the green color level),
#   blue  (number between 0 and 255 representing the blue color level),
#   alpha (number between 0 and 255 representing the alpha level),
# ]
ERRORMESSAGE
      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    return failed
  end

  def is_line_entry_valid? entry
    return false if !entry.is_a? Array
    return false if !(entry.length == 7 || entry.length == 4 || entry.length == 8)
    return !entry.any? { |e| !(e.is_a?(Fixnum) || e.is_a?(Float)) }
  end

  def report_line_item_not_correct_if_needed entry, index
    failed = !is_line_entry_valid?(entry)
    return failed if @root.paused

    if failed
      error_string = <<-ERRORMESSAGE
# Error `line` entry invalid:
# The entry at index #{index} is invalid:
# Type:   #{entry.class}
# Value:  #{entry}
#
# All entries within `lines` must be an array with 4, 7 or 8 values
# that conform to the following structure:
# [
#   x     (number representing the x position),
#   y     (number representing the y position),
#   x2    (number representing the width),
#   y2    (number representing the height),
#   red   (number between 0 and 255 representing the red color level),
#   green (number between 0 and 255 representing the green color level),
#   blue  (number between 0 and 255 representing the blue color level),
#   alpha (number between 0 and 255 representing the alpha level),
# ]
ERRORMESSAGE
      $layout.prepend_current_repl_code error_string.strip + "\n\n"
      $layout.queue_flash :failure, 0, 0, 1280, 720, *$layout.red, 1 / 20.0
    end

    return failed
  end

  def lines
    failed = report_lines_not_array_if_needed

    if failed
      @root.paused = true
      return [0, 0, 0, 0, 0, 0, 0, 0]
    end

    index = 0

    results = (@root.lines || []).map do |l|
      failed = report_line_item_not_correct_if_needed l, index

      if failed
        @root.paused = true
        [0, 0, 0, 0, 0, 0, 0, 0]
      else
        if l.length == 4
          [
            l[0].round.to_i,  # x
            l[1].round.to_i,  # y
            l[2].round.to_i,  # w
            l[3].round.to_i,  # h,
            0, 0, 0, 255
          ]
        elsif l.length == 7
          [
            l[0].round.to_i, # x
            l[1].round.to_i, # y
            l[2].round.to_i, # w
            l[3].round.to_i, # h
            l[4].round.to_i, # r
            l[5].round.to_i, # g
            l[6].round.to_i, # b
            255
          ]
        elsif l.length == 8
          [
            l[0].round.to_i, # x
            l[1].round.to_i, # y
            l[2].round.to_i, # w
            l[3].round.to_i, # h
            l[4].round.to_i, # r
            l[5].round.to_i, # g
            l[6].round.to_i, # b
            l[7].round.to_i, # a
          ]
        end
      end
    end

    return results
  end

  def buttons
    []
  end

  def progress_bars
    []
  end

  def sprites
    []
  end

  def sprite_buttons
    []
  end

  def tall_buttons
    []
  end

  def selection_frame
    [0, 0, 0, 0]
  end

  def selection_frame_two
    [0, 0, 0, 0]
  end

  def top_labels
    []
  end
end
