class GameView
  attr_accessor :cached_labels

  def frames_each_tick game
    if game.active_module == :world_event_battle
      100
    else
      44.5
    end
  end

  def initialize width, height
    @layout = Layout.new width, height
    @layout.parent_delegate = self
    $dragon = Dragon.new
  end

  def on_reload game
    @layout.on_reload game
  end

  def on_scene_changed id, selected_control, game
    invalidate_cache game, true
  end

  def on_startup game
    @layout.on_startup game
    @frame_background_color = [0, 0, 0]
    @target_background_color = @layout.background_color game
    @frame_text_color = [0, 0, 0]
    @target_text_color = @layout.text_color game
    @current_game_tick = game.tick_count
    @previous_game_tick = game.tick_count
    @ticks_between = 0
  end

  def tick game
    if game.active_module == :repl
      $dragon.calc_wrapper
    end
    @layout.tick game
    game.tick_frame
    @ticks_between += 1
    invalidate_cache game
  end

  def invalidate_cache game, force = false
    if @current_game_tick != game.tick_count
      @previous_game_tick = @current_game_tick
      @current_game_tick = game.tick_count
      @cached_labels = nil
      @ticks_between = 0
    end

    if force || @ticks_between == 0
      @cached_labels = nil
      @progress_bars = nil
      @progress_bar_previous_percentages = nil unless game.active_module == :world_event_battle
      progress_bars(game)
    end
  end

  def towards_number target, current, rate
    return target if target == current
    return target if (target - current).abs < rate
    return target if (target - current).abs < rate
    return current + rate if target > current
    return current - rate if target < current
  end

  def background_color game
    return $dragon.background_color if game.active_module == :repl

    @target_background_color = @layout.background_color game
    t_r, t_g, t_b = @target_background_color
    @frame_background_color[0] = towards_number t_r, @frame_background_color[0], 2
    @frame_background_color[1] = towards_number t_g, @frame_background_color[1], 2
    @frame_background_color[2] = towards_number t_b, @frame_background_color[2], 2
    @frame_background_color
  end

  def borders game
    return $dragon.borders if game.active_module == :repl
    @layout.borders game
  end

  def text_color game
    return $dragon.text_color if game.active_module == :repl

    @target_text_color = @layout.text_color game
    t_r, t_g, t_b = @target_text_color
    @frame_text_color[0] = towards_number t_r, @frame_text_color[0], 2
    @frame_text_color[1] = towards_number t_g, @frame_text_color[1], 2
    @frame_text_color[2] = towards_number t_b, @frame_text_color[2], 2
    @frame_text_color
  end

  def sounds game
    @layout.sounds game
  end

  def lines game
    return $dragon.lines if game.active_module == :repl
    @layout.lines game
  end

  def labels_with_alpha game
    return $dragon.labels if game.active_module == :repl
    []
  end

  def labels game
    return [] if game.active_module == :repl
    if game.active_module == :world_event_battle
      @layout.labels game
    else
      @cached_labels ||= @layout.labels game
      @cached_labels
    end
  end

  def sprites game
    return $dragon.sprites if game.active_module == :repl
    @layout.sprites game
  end

  def is_japanese game
    @layout.is_japanese game
  end

  def queue_flash  id, x, y, w, h, r, g, b, rate
    @layout.queue_flash id, x, y, w, h, r, g, b, rate
  end

  def highlights game
    if game.active_module == :repl
      begin
        return $dragon.highlights + @layout.highlights(game)
      rescue Exception => e
        $dragon.root.paused = true
        $layout.append_current_repl_code "# #{e}"
        return []
      end
    else
      return @layout.highlights game
    end
  end

  def interprolated_percent progress_bar, last_percentage, game
    current_percentage = progress_bar[5]
    return current_percentage / 100.0 if !progress_bar[6]
    total_ticks_to_reach_100 = (progress_bar[6] - 1)
    total_frames = (total_ticks_to_reach_100) * frames_each_tick(game)
    increase_per_frame = total_frames / total_ticks_to_reach_100
    percentage_increase_per_frame = (increase_per_frame / total_frames) * 100
    increment = current_percentage + ((@ticks_between / frames_each_tick(game)) * percentage_increase_per_frame)
    next_percentage = current_percentage + percentage_increase_per_frame
    final_result = 1.0
    if increment > next_percentage
      final_result = current_percentage / 100
    else
      final_result = increment / 100.0
    end

    if last_percentage && last_percentage > final_result && last_percentage < 1.0 && final_result < 1.0
      return last_percentage
    end

    return 1.0 if final_result > 1
    return final_result
  end

  def progress_bars game
    return $dragon.progress_bars if game.active_module == :repl
    if game.active_module == :world_event_battle
      @progress_bars = nil
      @progress_bar_previous_percentages = nil
    end

    index = 0
    @progress_bar_previous_percentages ||= {}
    @progress_bars ||= @layout.progress_bars game
    @progress_bars.map do |p|
      if p.length == 7
        interprolated_result = interprolated_percent(p, @progress_bar_previous_percentages[index], game)
        @progress_bar_previous_percentages[index] = interprolated_result
        index += 1
        # [p[0], p[1], p[2], p[3], "#{p[4]} #{p[5]} #{p[6]} #{interprolated_result}", interprolated_result]
        [p[0], p[1], p[2], p[3], p[4], interprolated_result]
      else
        puts "Progress bars in #{game.active_module} do not contain rate."
        raise "GameView#progress_bars"
      end
    end
  end

  def buttons game
    return $dragon.buttons if game.active_module == :repl

    @layout.buttons game
  end

  def sprite_buttons game
    return $dragon.sprite_buttons if game.active_module == :repl

    @layout.sprite_buttons game
  end

  def tall_buttons game
    return $dragon.tall_buttons if game.active_module == :repl

    @layout.tall_buttons game
  end

  def selection_frame game
    return $dragon.selection_frame if game.active_module == :repl

    @layout.selection_frame game
  end

  def selection_frame_two game
    return $dragon.selection_frame_two if game.active_module == :repl

    @layout.selection_frame_two game
  end

  def top_labels game
    return $dragon.top_labels if game.active_module == :repl

    @layout.top_labels game
  end

  def key_up_raw raw_key, modifier, game
    @cached_labels = nil
    @layout.key_up_raw raw_key, modifier, game
  end

  def key_pressed_player_one raw_key, game
    if game.active_module == :repl
      if raw_key == 0
        $dragon.root.player_1_accept = true
      elsif raw_key == 1
        $dragon.root.player_1_up = true
      elsif raw_key == 2
        $dragon.root.player_1_down = true
      elsif raw_key == 3
        $dragon.root.player_1_left = true
      elsif raw_key == 4
        $dragon.root.player_1_right = true
      end
    else
      @layout.key_pressed_player_one raw_key, game
    end
  end

  def key_pressed_player_two raw_key, game
    if game.active_module == :repl
      if raw_key == 0
        $dragon.root.player_2_accept = true
      elsif raw_key == 1
        $dragon.root.player_2_up = true
      elsif raw_key == 2
        $dragon.root.player_2_down = true
      elsif raw_key == 3
        $dragon.root.player_2_left = true
      elsif raw_key == 4
        $dragon.root.player_2_right = true
      end
    else
      @layout.key_pressed_player_two raw_key, game
    end
  end

  def mouse_pressed mousex, mousey, game
    # mousey -= 10
    if game.active_module == :repl
      $dragon.root.click_x         = mousex
      $dragon.root.click_y         = mousey
      $dragon.root.touch_x         = mousex
      $dragon.root.touch_y         = mousey
    else
      @layout.mouse_pressed mousex, mousey, game
    end
  end

  def repl_labels game
    @layout.repl_labels game
  end

  def key_down_player_one raw_key, game
    @layout.key_down_player_one raw_key, game
  end

  def key_down_player_two raw_key, game
    @layout.key_down_player_two raw_key, game
  end

  def key_up_player_one raw_key, game
    @layout.key_up_player_one raw_key, game
  end

  def key_up_player_two raw_key, game
    @layout.key_up_player_two raw_key, game
  end

  def key_held_player_one raw_key, game
    @layout.key_held_player_one raw_key, game
  end

  def key_held_player_two raw_key, game
    @layout.key_held_player_two raw_key, game
  end
end
