# coding: utf-8
class Layout
  include LayoutDebugging
  include LayoutAlert
  include LayoutWordLookup
  include LayoutHistory
  include LayoutSupplies
  include LayoutRoom
  include LayoutOutside
  include LayoutBuilder
  include LayoutTradepost
  include LayoutKeyPresses
  include LayoutEmbark
  include LayoutLoot
  include LayoutWorld
  include LayoutWorldEvent
  include LayoutWorldEventBattle
  include LayoutWorkshop
  include LayoutStarship
  include LayoutSpace
  include LayoutRandomEvent
  include LayoutPerks
  include LayoutPause
  include LayoutContraption
  include LayoutRepl

  attr_accessor :width, :height,
                :ticks, :seconds, :minutes,
                :font_to_show, :layout_debugging,
                :selected_control, :parent_delegate,
                :layout_key_pressed_column, :red_flash_perc,
                :flashes, :player_two_exists

  def initialize width, height
    @ffi_file = FFI::File.new
    @ffi_mrb = FFI::MRB.new
    $ffi = @ffi_mrb unless $ffi
    @width = width
    @height = height
    @last_history = []
    @last_history_result = []
    @sound_queue = []
    @selected_control = {}
    @selected_control[:one] = :room_fire_progress_bar
    @layout_debugging = false
    @help_text_debounce = 0
    @player_two_exists = false
    @player_two_debounce = 60 * 60 * 10
    @__locals = {} #for FFI stuff I copy pasta'ed from RetroRuby
    @ticks = 0
    @seconds = 0
    @minutes = 0
    @font_to_show = nil
    @supply_flashes = {}
    @supply_flashes_error = {}
    @flashes = {}
    __files_to_reload.each { |f| __init_mtimes f }
  end

  def toast message
    alert_show "",
               [message],
               $game.active_module,
               $game
  end

  def layout_selected_control_one
    @selected_control[:one]
  end

  def queue_supply_flash key
    @supply_flashes[key] = true
  end

  def queue_supply_flash_error key
    @supply_flashes_error[key] = true
  end

  def player_two_debounce!
    return unless @player_two_exists

    @player_two_debounce -= 1
    if @player_two_debounce < 0
      @player_two_exists = false
      @player_two_debounce = 60 * 60
    end
  end

  def help_text_debounce!
    @help_text_debounce = 10 * 60
  end

  def font_sm_sm_sm
    -2
  end

  def font_sm_sm
    -1
  end

  def font_sm
    0
  end

  def font_med
    1
  end

  def font_lg_lg
    2
  end

  def left_aligned
    0
  end

  def center_aligned
    1
  end

  def right_aligned
    2
  end

  def show_scene id, selected_control, game
  end

  def make_new_lines word, rest, length, aggregate
  end

  def japanese_word_length word
  end

  def make_new_lines_japanese word, rest, length, aggregate
  end

  def long_lines_split_japanese line, length
  end

  def long_lines_split line, length
  end

  def on_shown_or_dismissed game
  end

  def tick game
    $layout = self if !$layout

    @help_text_debounce -= 1 if @help_text_debounce > 0

    @ticks += 1

    tick_flashes

    if @ticks == 60
      @seconds += 1
      @ticks = 0
      if !game.is_production
        __files_to_reload.each { |f| __reload_if_needed f }
      end
    end

    if @seconds == 60
      @minutes += 1
      @seconds = 0
    end

    if @layout_previous_game_tick != game.tick_count
      @layout_key_pressed_cached_nav = nil
      @layout_previous_game_tick = game.tick_count
    end

    on_shown_or_dismissed game
    room_tick game
    outside_tick game
    space_tick game
    world_event_battle_tick game
    world_tick game
    contraption_tick game
    pause_tick game
    repl_tick game
    true
  end

  def borders game
    [
      *debug_guide_lines(game),
      history_border(game),
      supplies_border(game),
      *space_lines(game),
      outside_border(game),
      loot_borders(game),
      embark_border(game),
      *contraption_borders(game),
      *world_borders(game)
    ].reject { |i| i.nil? }
  end

  def top_labels game
    [
      *debug_guide_labels(game)
    ]
  end

  def no_movement_help_text game
  end

  def color_with_perc_towards r, g, b, perc, t_r, t_g, t_b
    [
      (r + (perc * (t_r - r))).round,
      (g + (perc * (t_g - g))).round,
      (b + (perc * (t_b - b))).round
    ]
  end

  def tick_flashes
    to_delete = []
    @flashes.each do |k, v|
      v[:current] += v[:rate]
      if v[:current] > 1.0
        to_delete << k
      end
    end

    to_delete.each { |k| @flashes.delete(k) }
  end

  def queue_flash id, x, y, w, h, r, g, b, rate
    @flashes[id] = {
      x: x.round,
      y: y.round,
      w: w.round,
      h: h.round,
      r: r,
      g: g,
      b: b,
      rate: rate,
      current: 0
    }
  end

  def highlights game
  end

  def queue_sound path
    @sound_queue << [get_base_path + path, 0, 0]
  end

  def queue_music path, repeat
    return if @current_music == path
    @sound_to_be_staged = path # TODO: Fade out before fading in.
    @sound_queue << [get_base_path + path, 1, repeat ? 1 : 0]
    @current_music = path
  end

  def stop_music
    @sound_queue << ['', 2, 0]
    @current_music = nil
  end

  def sounds game
    results = @sound_queue.to_a
    @sound_queue.clear
    return results
  end

  def repl_labels game
    [
      *__repl_labels(game)
    ].reject { |l| l.nil? }
  end

  def labels game
    # x, y, string, font size, centered
    [
      *no_movement_help_text(game),
      *room_labels(game),
      *outside_labels(game),
      *builder_labels(game),
      *tradepost_labels(game),
      *workshop_labels(game),
      *embark_labels(game),
      *loot_labels(game),
      *world_labels(game),
      *world_event_labels(game),
      *world_event_battle_labels(game),
      *history_labels(game),
      *alert_labels(game),
      *supplies_labels(game, 17),
      *starship_labels(game),
      *space_labels(game),
      *random_event_labels(game),
      *perks_labels(game),
      *pause_labels(game),
      *contraption_labels(game),
    ].reject { |l| l.nil? }
  end

  def sprites game
    # x, y, string, font size, centered
    [
      *world_sprites(game),
      *world_event_battle_sprites(game),
      *space_sprites(game)
    ].reject { |l| l.nil? }
  end

  def buttons game
    [
      *room_buttons(game),
      *outside_buttons(game),
      *builder_buttons(game),
      *tradepost_buttons(game),
      *embark_buttons(game),
      *loot_buttons(game),
      *workshop_buttons(game),
      *world_event_buttons(game),
      *world_event_battle_buttons(game),
      *starship_buttons(game),
      *random_event_buttons(game),
      *perks_buttons(game),
      *space_buttons(game),
      *pause_buttons(game),
    ].reject { |b| b.nil? }
  end

  def sprite_buttons game
    [
      *alert_buttons(game),
      *outside_sprite_buttons(game),
      *embark_sprite_buttons(game),
      *loot_sprite_buttons(game),
      *contraption_buttons(game)
    ].reject { |b| b.nil? }
  end

  def tall_buttons game
    [
      *builder_tall_buttons(game),
      *tradepost_tall_buttons(game),
      *workshop_tall_buttons(game)
    ].reject { |b| b.nil? }
  end

  def progress_bars game
    [
      *room_progress_bars(game),
      *outside_progress_bars(game),
      *builder_progress_bars(game),
      *embark_progress_bars(game),
      *world_event_battle_progress_bars(game),
      *starship_progress_bars(game)
    ].reject { |b| b.nil? }
  end

  def get_combinations input, size
    (size - 1)
      .times
      .reduce(input) { |aggregate, _| aggregate.product(input) }
      .map(&:flatten)
      .reject { |a| a.uniq.length != a.length }
  end

  def lines game
    [
      *world_lines(game)
    ].reject { |l| l.nil? }
  end

  def light_gray
    [50, 50, 50]
  end

  def really_light_gray
    [200, 200, 200]
  end

  def gray
    [175, 175, 175]
  end

  def green
    [15, 150, 90]
  end

  def red
    [200, 50, 50]
  end

  def white
    [240, 240, 240]
  end

  def black
    [0, 0, 0]
  end

  def dark_gray
    [20, 20, 20]
  end

  def text_color game
  end

  def background_color game
  end

  def controller_nice_name_for_raw raw
    @raw_controller_key_lookup ||= {
      0   => :a,
      1   => :up,
      2   => :down,
      3   => :left,
      4   => :right,
      13  => :start,
      14  => :b,
      15  => :a,
      16  => :x,
      17  => :y,
      18  => :l1,
      19  => :r1,
      20  => :l3,
      21  => :r3,
      22  => :l2,
      23  => :r2,
    }

    @raw_controller_key_lookup[raw]
  end

  def set_key raw_key, playernum, event = nil
    if event
      event = "_#{event}"
    end

    is_key_up = !event

    controller_nice_name = controller_nice_name_for_raw raw_key
    key = nil
    if controller_nice_name
      if playernum == 1
        key = "player_1_#{controller_nice_name}#{event}".to_sym
        $dragon.root.hash[key] ||= $dragon.root.tick_count
        if is_key_up
          $dragon.root.hash["player_1_#{controller_nice_name}_key_down".to_sym] = false
          $dragon.root.hash["player_1_#{controller_nice_name}_key_held".to_sym] = false
        end
      elsif playernum == 2
        key = "player_2_#{controller_nice_name}#{event}".to_sym
        $dragon.root.hash[key] ||= $dragon.root.tick_count
        if is_key_up
          $dragon.root.hash["player_2_#{controller_nice_name}_key_down".to_sym] = false
          $dragon.root.hash["player_2_#{controller_nice_name}_key_held".to_sym] = false
        end
      else
        # puts "Key down for playernum `#{playernum}\" raw key `#{raw_key}\" not found."
      end
    else
      # puts "Key down for playernum `#{playernum}\" raw key `#{raw_key}\" not found."
    end
  end

  def key_down_player_one raw_key, game
    set_key raw_key, 1, :key_down
  end

  def key_down_player_two raw_key, game
    set_key raw_key, 2, :key_down
  end

  def key_up_player_one raw_key, game
    set_key raw_key, 1
    if controller_nice_name_for_raw(raw_key) == :a
      $dragon.root.player_1_accept = true
    end
  end

  def key_up_player_two raw_key, game
    set_key raw_key, 2
    if controller_nice_name_for_raw(raw_key) == :a
      $dragon.root.player_2_accept = true
    end
  end

  def key_held_player_one raw_key, game
    set_key raw_key, 1, :key_held
  end

  def key_held_player_two raw_key, game
    set_key raw_key, 2, :key_held
  end
end
