module LayoutRepl
  def repl_shown game
    @repl_save_path = game.base_save_path
    @current_repl_code_index = 0
    @all_repl_code = { }
    @selected_control[:one] = nil
    @selected_control[:two] = nil
    set_current_repl_code repl_instructions
    @cursor_index = 43
    stop_music
    10.times { |i| repl_load i }
  end

  def in_game_repl_shown
    @in_game_repl_shown
  end

  def in_game_repl_shown= value
    @in_game_repl_shown = value
  end

  def repl_instructions
    <<-INSTRUCTIONS
# http://amirrajan.net/power-overwhelming


    INSTRUCTIONS
  end

  def repl_load index
    @all_repl_code ||= { }
    if File.exists? repl_file_name(index)
      @all_repl_code[index] = IO.read(repl_file_name(index))
    end
  end

  def get_repl_code index
    index = 0 if index < 0
    index = 9 if index > 9
    @all_repl_code ||= { }
    @all_repl_code[index] || ""
  end

  def prepend_buffer index, text
    index = 0 if index < 0
    index = 9 if index > 9
    @all_repl_code ||= { }
    @all_repl_code[index] ||= ""
    @all_repl_code[index].prepend text
  end

  def prepend_current_repl_code text
    prepend_buffer @current_repl_code_index, text
  end

  def append_buffer index, text
    append_repl_code index, text
  end

  def set_buffer index, text
    set_repl_code index, text
  end

  def get_buffer index
    get_buffer_text index
  end

  def append_repl_code index, text
    index = 0 if index < 0
    index = 9 if index > 9
    @all_repl_code ||= { }
    @all_repl_code[index] ||= ""
    @all_repl_code[index] << text
  end

  def set_repl_code index, text
    index = 0 if index < 0
    index = 9 if index > 9
    @all_repl_code ||= { }
    @all_repl_code[index] = text
  end

  def append_current_repl_code value
    @all_repl_code ||= { }
    @current_repl_code_index ||= 0
    @all_repl_code[@current_repl_code_index] << value
  end

  def set_current_repl_code value
    @all_repl_code ||= { }
    @current_repl_code_index ||= 0
    @all_repl_code[@current_repl_code_index] = value
  end

  def get_current_repl_code
    @all_repl_code ||= { }
    @current_repl_code_index ||= 0
    if (@all_repl_code[@current_repl_code_index] || "").length == 0
      @all_repl_code[@current_repl_code_index] = repl_instructions
      @cursor_index = 43
    end
    @all_repl_code[@current_repl_code_index]
  end

  def repl_cursor_index
    @cursor_index
  end

  def repl_set_cursor location
    if location > get_current_repl_code.length
      @cursor_index = get_current_repl_code.length
    else
      @cursor_index = location
    end
  end

  def repl_tick game
    if game.active_module == :repl
      blink_cursor game
    end
    if repl_enabled?(game)
      blink_cursor game
    end
    if mini_buffer_enabled?
      blink_cursor game
    end
  end

  def repl_dismissed game

  end

  def repl_code_line i
    repl_code_lines[i] || ""
  end

  def repl_code_lines args = nil
    args ||= {}
    target_line = :max
    drop = 0

    if args[:til] || args[:till]
      target_line = args[:til] || args[:till]
    elsif args[:find]
      target_line = args[:find] + 1
    end

    if args[:drop]
      drop = args[:drop]
    end

    if target_line == :max
      get_current_repl_code.split("\n", -1).drop(drop) || []
    elsif target_line <= 0
      []
    else
      get_current_repl_code.split("\n", -1).drop(drop).take(target_line) || []
    end
  end

  def repl_cursor_up
    @cursor_index ||= 0
    x, y = repl_ordinal_cursor
    if y == 0
      @cursor_index = 0
    else
      lines = repl_code_lines({ til: y - 1 })
      if lines.length == 0
        lines_length = 0
      else
        lines_length = lines.join("\n").length
      end
      @cursor_index = lines_length
      _, y = repl_ordinal_cursor
      current_line = repl_code_line y
      if current_line.length < x
        @cursor_index += current_line.length
      else
        if y == 0
          @cursor_index = (x - 1)
          @cursor_index = 0 if @cursor_index < 0
        else
          @cursor_index += x
        end
      end
    end
  end

  def repl_cursor_down
    @cursor_index ||= 0
    x, y = repl_ordinal_cursor
    lines = repl_code_lines({ find: y })
    if lines.length == 0
      lines_length = 0
    else
      lines_length = lines.join("\n").length
    end

    if lines_length + x > get_current_repl_code.length
      @cursor_index = lines_length
    else
      @cursor_index = lines_length + x
    end
  end

  def repl_cursor_left
    repl_set_cursor @cursor_index - 1
  end

  def repl_cursor_right
    repl_set_cursor @cursor_index + 1
  end

  def repl_enabled? game
    return true if game.active_module == :repl
    return true if @in_game_repl_shown
    return false
  end

  def char_with_shift raw_key, modifier, game
    return nil unless raw_key >= 0 && raw_key <= 255
    if modifier != 1 && modifier != 2 && modifier != 3
      return raw_key.chr
    else
      @shift_keys ||= {
        '`' => '~', '-' => '_', "'" => '"', "1" => '!',
        "2" => '@', "3" => '#', "4" => '$', "5" => '%',
        "6" => '^', "7" => '&', "8" => '*', "9" => '(',
        "0" => ')', ";" => ":", "=" => "+", "[" => "{",
        "]" => "}", '\\'=> "|", '/' => "?", '.' => ">",
        ',' => "<", 'a' => 'A', 'b' => 'B', 'c' => 'C',
        'd' => 'D', 'e' => 'E', 'f' => 'F', 'g' => 'G',
        'h' => 'H', 'i' => 'I', 'j' => 'J', 'k' => 'K',
        'l' => 'L', 'm' => 'M', 'n' => 'N', 'o' => 'O',
        'p' => 'P', 'q' => 'Q', 'r' => 'R', 's' => 'S',
        't' => 'T', 'u' => 'U', 'v' => 'V', 'w' => 'W',
        'x' => 'X', 'y' => 'Y', 'z' => 'Z'
      }

      @shift_keys[raw_key.chr.to_s] || raw_key.chr.to_s
    end
  end

  def repl_cursor_key
    repl_ordinal_cursor
    "repl_ibeam_#{@cursor_index}".to_sym
  end

  def repl_ordinal_cursor
    return [0, 0] if get_current_repl_code.length == 0
    @cursor_index ||= get_current_repl_code.length - 1
    code_point = get_current_repl_code[0..@cursor_index]
    splits = code_point.split("\n", -1)
    x = splits.last.length
    y = splits.length - 1
    [x, y]
  end

  def repl_cursor_width
    3
  end

  def repl_cursor_location
    if mini_buffer_enabled?
      ibeam_location_x, ibeam_location_y, text = mini_buffer_label
      ibeam_location_x += text.length * 9.92
    else
      x, y = repl_ordinal_cursor
      ibeam_location_x = 60 + x * 9.92
      ibeam_location_y = 40 + y * 23
    end

    [ibeam_location_x, ibeam_location_y, repl_cursor_width, 24]
  end

  def repl_cursor_location_shifted game
    x, y, w, h = repl_cursor_location
    if repl_enabled? game
      y -= (y - 360) if y > 360
    else
      y -= 27
    end

    [x, y, w, h]
  end

  def blink_cursor game
    should_blink = false
    if game.active_module == :repl
      should_blink ||= true
    end
    if repl_enabled? game
      should_blink ||= true
    end
    if mini_buffer_enabled?
      should_blink ||= true
    end
    return unless should_blink
    key = repl_cursor_key
    unless @flashes[key]
      queue_flash key, *repl_cursor_location_shifted(game), 100, 100, 200, 1 / 40.0
    end
  end

  def repl_line_numbers game
    set_current_repl_code(get_current_repl_code || "")
    running_y = 65
    _, y, _, _ = repl_cursor_location

    if y > 360
      running_y -= (y - 360)
    end

    index = 1
    running_cursor_index = 0
    labels = get_current_repl_code.each_line.map do |l|
      text = [10, running_y, "#{index}", 0, 0, 150, 150, 150, 1]
      running_y += 23
      index += 1
      text
    end
    labels
  end

  def repl_code_labels game
    set_current_repl_code(get_current_repl_code || "")
    running_y = 65
    _, y, _, _ = repl_cursor_location

    if y > 360
      running_y -= (y - 360)
    end

    running_cursor_index = 0
    labels = get_current_repl_code.each_line.map do |l|
      text = [60, running_y, l + "\n", 0, 0, 0, 0, 0, 1]
      running_y += 23
      text
    end
    labels
  end

  def bye
    $game.active_module = $game.previous_active_module
  end

  def repl_clear
    @cursor_index = 0
    set_current_repl_code ""
  end

  def in_game_repl_labels game
    return unless repl_enabled? game

    [
      *repl_line_numbers(game),
      *repl_code_labels(game),
      [5, 20,  "buffer: #{(@current_repl_code_index).to_s}", -2, 0, 0, 0, 0, 1]
    ]
  end

  def __repl_labels game
    [
      *in_game_repl_labels(game),
      mini_buffer_label
    ].reject { |l| l.nil? }
  end

  def repl_eval_buffer raw_key, char, modifier
    if raw_key == 13 && (modifier == 1 || modifier == 2 || modifier == 3)
      begin
        result = $ffi.parse get_current_repl_code
        if result != 'Syntax OK'
          result.strip.split("\n").each do |l|
            append_current_repl_code "\n# #{l}"
          end
          queue_flash :failure, 0, 0, 1280, 720, *red, 1 / 20.0
        else
          $dragon.root.paused = false
          $ffi.eval get_current_repl_code
          queue_flash :success, 0, 0, 1280, 720, *green, 1 / 20.0
        end
      rescue Exception => e
        $dragon.root.paused = true
        append_current_repl_code "\n # #{e}"
        queue_flash :failure, 0, 0, 1280, 720, *red, 1 / 20.0
      end
    else
      before_cursor, after_cursor = repl_before_after_cursor
      set_current_repl_code (before_cursor + "\n" + after_cursor)
      @cursor_index += 1
    end

    repl_save @current_repl_code_index
  end

  def repl_save index
    File.open(repl_file_name(index), "w") do |f|
      f.write get_repl_code(index)
    end

    commit_save_data
  rescue
    puts "save file failed"
  end

  def repl_file_name index
    "#{@repl_save_path}repl-buffer-#{index}.txt"
  end

  def repl_before_after_cursor
    before_cursor = (get_current_repl_code.slice(0..@cursor_index)  || "")
    after_cursor  = (get_current_repl_code.slice(@cursor_index + 1..-1) || "")
    [before_cursor, after_cursor]
  end

  def process_repl raw_key, char, modifier, game
    $repl ||= self
    @layout_repl_cursor = 0
    if raw_key == 9 # tab
      if raw_key == 9 && (modifier == 1 || modifier == 2 || modifier == 3)
        @current_repl_code_index -= 1
        if @current_repl_code_index < 0
          @current_repl_code_index = 9
        end
      else
        @current_repl_code_index += 1
        if @current_repl_code_index > 9
          @current_repl_code_index = 0
        end
      end
    elsif raw_key == 8 # delete
      if get_current_repl_code.length == 1
        set_current_repl_code ""
        @cursor_index = 0
      else
        if (modifier == 1 || modifier == 2 || modifier == 3)
          x, y = repl_ordinal_cursor
          before_lines = repl_code_lines({ til: y })
          after_lines  = repl_code_lines({ drop: y + 1 })
          set_current_repl_code (before_lines.join("\n") + "\n" + after_lines.join("\n")).strip
          if y == 0
            @cursor_index = 0
          else
            @cursor_index -= x
          end

          if get_current_repl_code.strip.length == 0
            @cursor_index = 0
          elsif @cursor_index >= (get_current_repl_code.length - 1)
            @cursor_index -= 1
          end
        else
          before_cursor, after_cursor = repl_before_after_cursor
          set_current_repl_code (before_cursor[0, before_cursor.length - 1] + after_cursor)
          @cursor_index -= 1
          if @cursor_index < 0
            @cursor_index = 0
          end
          if @cursor_index > get_current_repl_code.length
            @cursor_index = get_current_repl_code.length
          end
        end
      end
    elsif raw_key == 13 # enter
      repl_eval_buffer raw_key, char, modifier
    else
      before_cursor, after_cursor = repl_before_after_cursor
      set_current_repl_code (before_cursor + char + after_cursor)
      @cursor_index += 1
    end

    blink_cursor game
  end

  def mini_buffer_enabled?
    @in_game_mini_buffer_shown
  end

  def show_cursor? game
    repl_enabled?(game) || mini_buffer_enabled?
  end

  def show_repl game
    @in_game_mini_buffer_shown = false
    @mini_buffer_code = ""
    @in_game_repl_shown = true
    game.active_module = :repl
    repl_shown game
    blink_cursor game
  end

  def key_up_raw_show_hide_mini_buffer raw_key, modifier, game
    return if repl_enabled? game
    return unless raw_key >= 0 && raw_key <= 255 && raw_key != 27
    char = char_with_shift raw_key, modifier, game
    if (char == ':' || char == '$') && !mini_buffer_enabled?
      set_mini_buffer_code ":"
      if char == '$'
        append_mini_buffer_code "$dragon."
      end
      @in_game_mini_buffer_shown = true
    else
      if raw_key >= 0 && raw_key <= 255 && raw_key != 27
        append_mini_buffer_code char
      end
      process_mini_buffer raw_key, char, modifier, game
    end

    blink_cursor game
  end

  def key_up_raw_show_hide_repl raw_key, modifier, game
    return unless raw_key >= 0 && raw_key <= 255

    char = char_with_shift raw_key, modifier, game

    if (char == '`' || char == '~') && !repl_enabled?(game)
      show_repl game
    elsif (raw_key == 27) && repl_enabled?(game)
      hide_repl game
    elsif repl_enabled?(game)
      if @in_game_repl_shown
        process_repl raw_key, char, modifier, game
      end
    end
  end

  def key_up_raw_repl raw_key, modifier, game
    return if !repl_enabled?(game)

    if @in_game_repl_shown
      if    raw_key == 1073741906 # up
        repl_cursor_up
      elsif raw_key == 1073741905 # down
        repl_cursor_down
      elsif raw_key == 1073741904 # left
        repl_cursor_left
      elsif raw_key == 1073741903 # right
        repl_cursor_right
      end
    end
  end

  def key_up_raw raw_key, modifier, game
    key_up_raw_show_hide_repl raw_key, modifier, game
    key_up_raw_show_hide_mini_buffer raw_key, modifier, game
    key_up_raw_repl raw_key, modifier, game
    blink_cursor game
  end

  def hide_repl game
    repl_save @current_repl_code_index
    @in_game_repl_shown = false
    game.active_module = :room
  end

  def append_mini_buffer_code value
    set_mini_buffer_code "#{get_mini_buffer_code}#{value}"
  end

  def get_mini_buffer_code
    @mini_buffer_code ||= ""
    @mini_buffer_code
  end

  def set_mini_buffer_code value
    @mini_buffer_code ||= ""
    @mini_buffer_code = value
    @mini_buffer_code = @mini_buffer_code.gsub("\n", " ")
  end

  def process_mini_buffer raw_key, char, modifier, game
    return unless mini_buffer_enabled?
    if raw_key == 8 #delete
      if get_mini_buffer_code.strip.length == 1 || get_mini_buffer_code.strip.length == 0
        exit_mini_buffer_if_needed
      else
        set_mini_buffer_code(get_mini_buffer_code.strip[0...-2])
      end
    elsif raw_key == 13 # enter
      repl_eval_mini_buffer raw_key, char, modifier
    end
  end

  def mini_buffer_label
    return unless mini_buffer_enabled?
    [10, 30, "#{@mini_buffer_code}", 0, 0, 0, 0, 0, 1]
  end

  def repl_eval_mini_buffer raw_key, char, modifier
    if raw_key == 13
      begin
        code_to_run = get_mini_buffer_code.strip.to_s[1..-1]
        set_mini_buffer_code ":#{code_to_run}"
        result = $ffi.parse code_to_run
        if result != 'Syntax OK'
          result.strip.split("\n").each do |l|
            puts l
          end
          queue_flash :failure, 0, 0, 1280, 720, *red, 1 / 20.0
        else
          $ffi.eval code_to_run
          set_mini_buffer_code ""
          @in_game_mini_buffer_shown = false
          queue_flash :success, 0, 0, 1280, 720, *green, 1 / 20.0
        end
      rescue Exception => e
        $dragon.pause!
        queue_flash :failure, 0, 0, 1280, 720, *red, 1 / 20.0
      end
    end
  end

  def exit_mini_buffer_if_needed
    return if get_mini_buffer_code.strip.length > 2

    set_mini_buffer_code ""
    @in_game_mini_buffer_shown = false
  end
end
