module LayoutSpace
  def space_nav
    [:space_the_end_button, :space_crashed_button]
  end

  def space_tick game
    return unless game.active_module == :space
    return unless @space

    if !@ship_crashed
      @space.tick @space_ship_location_x - 500

      collision = false

      x, y, _, _ = space_ship_label game

      space_ship = [x, y, 10, 10]

      space_asteroid_labels(game).each do |l|
        l_x, l_y, _, _ = l
        if @space.collision? space_ship, [l_x, l_y, 10, 10]
          collision = true
        end
      end

      if !collision && !@previous_collision
        @previous_collision = false
      elsif !collision && @previous_collision
        @previous_collision = false
      elsif collision && !@previous_collision
        @hull_damage += 1
        queue_flash :collision_with_asteroid, 0, 0, 1280, 720, 255, 0, 0, 1 / 30.0
        @previous_collision = true
        if @hull_damage > @ship_hull
          @ship_crashing = true
        end
      elsif collision && @previous_collision
        @previous_collision = true
      end
    end

    if @space.completed?
      @ship_y_acceleration += 1 if @ship_y_acceleration < 100
      @space_ship_location_y -= @ship_y_acceleration
    elsif @ship_crashing
      @ship_y_acceleration += 1 if @ship_y_acceleration < 100
      @space_ship_location_y += @ship_y_acceleration
      if @space_ship_location_y > 10000
        @ship_crashed = true
      end
    else
      @ship_y_acceleration += 1 if @ship_y_acceleration < 100
      @space_ship_location_y -= @ship_y_acceleration
      if @space_ship_location_y < 550
        @space_ship_location_y = 550
        @ship_y_acceleration = 0
      end
    end
  end

  def space_descriptions game
    if game.solo_run?
      @space_descriptions ||= {
        1 =>  Translations.value("the sky. black. taunting."),
        10 => Translations.value("the air begins to thin. hard to breathe."),
        15 => Translations.value("temperature drops. the ship's windows are cool to the touch."),
        20 => Translations.value("the horizon. dark. desolate. the windows begin to freeze."),
        25 => Translations.value("she touches the walls. the ship begins to warm."),
        30 => Translations.value("the ship sputters, as if it's gasping for air."),
        35 => Translations.value("she whispers to it. the engines begin roar."),
        40 => Translations.value("she looks over. says goodbye..."),
        45 => Translations.value("she wells up with tears as a wall of asteroids approach."),
        50 => Translations.value("hands extended, she wills the asteroids away."),
        55 => Translations.value("her eyes focus on each rock, diverting their path."),
        60 => Translations.value("blood drips from her ears and nose.")
      }
    else
      @space_descriptions ||= {
        1 =>  Translations.value("the sky. black. taunting."),
        10 => Translations.value("the air begins to thin. hard to breathe."),
        15 => Translations.value("temperature drops. the ship's windows are cool to the touch."),
        20 => Translations.value("the horizon. dark. desolate. the windows begin to freeze."),
        25 => Translations.value("the scars of history, can be seen on the planet's surface."),
        30 => Translations.value("the starship sputters, as if it's gasping for air."),
        35 => Translations.value("this world, doesn't want anyone to escape."),
        40 => Translations.value("must be free of this prison. must break through."),
        45 => Translations.value("a wall of asteroids... heart begins to sink."),
        50 => Translations.value("so cold... so cold..."),
        55 => Translations.value("thoughts wander to the fire."),
        60 => Translations.value("through the iced windows, the lonely planet fades into the distance.")
      }
    end

    @space_descriptions
  end

  def space_dismissed game

  end

  def space_shown game
    stop_music
    @space_descriptions = nil
    space_descriptions(game)
    @supply_flashes.clear
    @supply_flashes_error.clear
    @flashes.clear
    @space_ship_location_x = 640
    @space_ship_location_y = 800
    @space = Space.new
    @current_collision = false
    @previous_collision = false
    @space.veer = game.outside.solo_run?
    @hull_damage = 0
    @ship_thrusters = game.ship_thrusters + 1
    @ship_hull = game.ship_hull
    @space_previous_label_text = nil
    @current_label_text = nil
    @space_story_line = nil
    @ship_y_acceleration = 0
    @ship_crashed = false
    @ship_crashing = false
  end

  def layout_move_space_left game
    return unless game.active_module == :space
    @space_ship_location_x -= @ship_thrusters
    if @space_ship_location_x < 500
      @space_ship_location_x = 500
    end
  end

  def layout_move_space_right game
    return unless game.active_module == :space
    @space_ship_location_x += @ship_thrusters
    if @space_ship_location_x > 750
      @space_ship_location_x = 750
    end
  end

  def space_buttons game
    return [] unless game.active_module == :space

    [space_the_end_button(game),
     space_crashed_button(game)].reject { |b| b.nil? }
  end

  def space_the_end_button_shown? game
    return false unless game.active_module == :space
    return false unless @space_ship_location_y
    return false unless @space_ship_location_y < -110000
    return false if @ship_crashed
    return true
  end

  def space_the_end_button game
    return unless space_the_end_button_shown? game

    @selected_control[:one] = :space_the_end_button
    @selected_control[:two] = :space_the_end_button
    if game.solo_run?
      if Translations.language == :japanese
        [550, 300, 180, 40, Translations.value("the end")]
      else
        [570, 300, 140, 40, Translations.value("the end")]
      end
    else
      if Translations.language == :japanese
        [550, 300, 180, 40, Translations.value("the end")]
      else
        [570, 300, 140, 40, Translations.value("the end?")]
      end
    end
  end

  def space_crashed_button game
    return nil unless game.active_module == :space
    return nil unless @ship_crashed

    @selected_control[:one] = :space_crashed_button
    @selected_control[:two] = nil
    [570, 600, 150, 40, Translations.value(" crashed")]
  end

  def space_press_end_button game
    game.save
    game.reset
    show_scene :room, :room_fire_progress_bar, game
    return true
  end

  def space_press_crash_button game
    game.history << "crashed."
    game.current_lift_off_ticks = 0
    show_scene :starship, :starship_back_button, game
    return true
  end

  def space_accept_pressed game, player
    if @selected_control[player] == :space_the_end_button
      return space_press_end_button game
    end

    if @selected_control[player] == :space_crashed_button
      return space_press_crash_button game
    end

    return false
  end

  def space_labels game
    return unless game.active_module == :space

    faded_color = [0, 0, 127, 127, 127]

    story_line = space_descriptions(game)[@space.altitude]

    @space_story_line = story_line if story_line

    huts_line =  "try completing without building huts."

    if game.solo_run?
      huts_line = ""
    end

    alignment = center_aligned
    starting_x = 632
    tom_and_tool_chain_labels = [
      [starting_x, 590,                    "          mruby hax0r : tom black           ",  1, alignment],
      [starting_x, 620, Translations.value(                  "TOOLCHAIN"                 ), 1, alignment],
      [starting_x, 650, Translations.value("                                          emacs+evil, tmux, iterm, mbp, bfo-9000"), 1, alignment],
    ]

    vote_for_pedro = [starting_x, 710,                    "                        pedro villarruel    " , 1, alignment]

    if game.language == :japanese
      alignment = left_aligned
      starting_x = 400
      tom_and_tool_chain_labels = [
        [starting_x, 590,                    "mruby hax0r : tom black           ",  1, alignment],
        [starting_x, 620, Translations.value(                  "TOOLCHAIN"                 ), 1, alignment],
        [starting_x, 650, Translations.value("       emacs+evil, tmux, iterm, mbp, bfo-9000"), 1, alignment],
      ]
      vote_for_pedro = [starting_x, 710,                    "                     pedro villarruel    " , 1, alignment]
    end



    # yes, I am using the ships location as a timer... T_T
    if @space_ship_location_y < -110000
      [
        [640, 40, Translations.value("a dark room"), 1, center_aligned],
        [640, 100, Translations.value("sired by michael townsend - @continuities."), 1, center_aligned],
        [640, 130, Translations.value("raised by amir rajan - @amirrajan."), 1, center_aligned],
        [640, 230, Translations.value("play time:") +
                     " #{((@this_run_completion_time || 0) / 60.0).round} " +
                     Translations.value("minutes"), 1, center_aligned],
        [640, 260, Translations.value("fastest time:") +
                     " #{((game.fastest_completion_ticks || 0) / 60.0).round} " +
                     Translations.value("minutes"), 1, center_aligned],
        [640, 290, Translations.value(huts_line), 1, center_aligned],
        [starting_x, 410, Translations.value("            publisher : circle entertainment"), 1, alignment],
        [starting_x, 440, Translations.value("    switch specialist : ryan c. gordon      "), 1, alignment],
        [starting_x, 470, Translations.value("apprentice programmer : brynnon walker      "), 1, alignment],
        [starting_x, 500, Translations.value("       sound director : rafael langoni smith"), 1, alignment],
        [starting_x, 530, Translations.value("       sound designer : arthur cordeiro     "), 1, alignment],
        [starting_x, 560, Translations.value("           translator : chie koizumi        "), 1, alignment],
        *tom_and_tool_chain_labels,
        [starting_x, 680, Translations.value("    special thanks to : sigono inc.         "), 1, alignment],
        vote_for_pedro,
      ]
    elsif @space_ship_location_y < -90000
      [
        [640, 40,  Translations.value("a dark room"), 1, center_aligned],
        [640, 100, Translations.value("sired by michael townsend - @continuities."), 1, center_aligned],
        [640, 130, Translations.value("raised by amir rajan - @amirrajan."), 1, center_aligned],
        [starting_x, 410, Translations.value("            publisher : circle entertainment"), 1, alignment],
        [starting_x, 440, Translations.value("    switch specialist : ryan c. gordon      "), 1, alignment],
        [starting_x, 470, Translations.value("apprentice programmer : brynnon walker      "), 1, alignment],
        [starting_x, 500, Translations.value("       sound director : rafael langoni smith"), 1, alignment],
        [starting_x, 530, Translations.value("       sound designer : arthur cordeiro     "), 1, alignment],
        [starting_x, 560, Translations.value("           translator : chie koizumi        "), 1, alignment],
        *tom_and_tool_chain_labels,
        [starting_x, 680, Translations.value("    special thanks to : sigono inc.         "), 1, alignment],
        vote_for_pedro,
      ]
    elsif @space_ship_location_y < -60000
      [
        [640, 40,  Translations.value("a dark room"), 1, center_aligned],
        [640, 100, Translations.value("sired by michael townsend - @continuities."), 1, center_aligned],
        [640, 130, Translations.value("raised by amir rajan - @amirrajan."), 1, center_aligned]
      ]
    elsif @space_ship_location_y < -30000
      [
        [640, 40, Translations.value("a dark room"), 1, center_aligned]
      ]
    elsif @space_ship_location_y < 0
      if game.solo_run?
        [ [640, 600, Translations.value("eyes rolling back, she collapses. stops breathing. the locket glows..."), 0, center_aligned] ]
      else
        [ [640, 600, Translations.value("shivering. gasping for air. must find her. the locket glows. must find her..."), 0, center_aligned] ]
      end
    elsif @space.completed?
      if !@has_logged_completion_time
        game.log_fastest_ticks
        @this_run_completion_time = game.tick_count
        @has_logged_completion_time = true
      end
      []
    elsif @ship_crashed || @ship_crashing
      []
    elsif @space_story_line
      [[20,  712, Translations.value("<> to pilot starship."), *faded_color],
       [640, 600, @space_story_line, 0, center_aligned],
       [640, 700, Translations.value("hull:") + " #{@ship_hull - @hull_damage}", 0, center_aligned] ]
    else
      [[20,  712, Translations.value("<> to pilot starship."), *faded_color],
       [640, 700, Translations.value("hull:") + " #{@ship_hull - @hull_damage}", 0, center_aligned] ]
    end
  end

  def space_sprites game
    return unless game.active_module == :space

    if @space && @space.asteroids
      [space_ship_label(game),
      *space_asteroid_labels(game)]
    else
      [space_ship_label(game)]
    end
  end

  def space_lines game
    return unless game.active_module == :space

    []
  end

  def space_asteroid_labels game
    return [] if @ship_crashed

    to_remove = @space.asteroids.find_all { |a| a.y > 800 }

    to_remove.each { |a| @space.remove_asteroid a }

    @space.asteroids.map { |a| [a.x.to_i + 490, a.y.to_i, "o"]  }
  end

  def space_ship_label game
    [@space_ship_location_x, @space_ship_location_y, "#"]
  end

  def layout_space_tap_maybe game
    if @ship_crashed
      space_press_crash_button game
    elsif space_the_end_button_shown? game
      space_press_end_button game
    end
  end
end
