module LayoutWorld
  def world_nav
    []
  end

  def world_dismissed game

  end

  def world_lines game
    return nil unless game.active_module == :world

    [
      *world_compass_line(game)
    ]
  end

  def compass_line_start_x
    1250
  end

  def compass_line_start_y
    670
  end

  def compass_size
    40
  end

  def compass_offset
    2
  end

  def world_compass_line game
    return nil unless game.active_module == :world
    x = compass_line_start_x
    y = compass_line_start_y
    c = compass_size
    ch = (c.to_f / 2.0).round.to_i #compass half length
    o = compass_offset

    [
      [x     , y     , x     , y + c ,          *text_color(game), 255],
      [x - ch, y + ch, x + ch, y + ch,          *text_color(game), 255],
      [x - ch, y +  o, x + c - 20 , y + c - o , *text_color(game), 255],
      [x + ch, y +  o, x - c + 20 , y + c - o , *text_color(game), 255],
    ]
  end

  def world_borders game
    return nil unless game.active_module == :world
    x  = compass_line_start_x - 2
    y  = compass_line_start_y - 2
    s  = compass_size  # compass size
    sh = (s.to_f / 2.0).round.to_i #compass half length # compass half size
    o  = compass_offset   # compass offset
    n_compass_dot_x  = x
    n_compass_dot_y  = y
    ne_compass_dot_x = x + sh
    ne_compass_dot_y = y +  o
    nw_compass_dot_x = x - sh
    nw_compass_dot_y = y +  o
    s_compass_dot_x  = x
    s_compass_dot_y  = y +  s
    se_compass_dot_x = x + sh
    se_compass_dot_y = y +  s - o
    sw_compass_dot_x = x - sh
    sw_compass_dot_y = y +  s - o
    e_compass_dot_x  = x + sh
    e_compass_dot_y  = y + sh
    w_compass_dot_x  = x - sh
    w_compass_dot_y  = y + sh
    @world_compass_dot_x ||= n_compass_dot_x
    @world_compass_dot_y ||= n_compass_dot_y
    @world_target_compass_dot_x ||= n_compass_dot_x
    @world_target_compass_dot_y ||= n_compass_dot_y
    @cached_compass_direction   ||= game.world.compass_direction

    if @cached_compass_direction == Translations.value('N') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 8 == 0)
      @world_target_compass_dot_x = n_compass_dot_x
      @world_target_compass_dot_y = n_compass_dot_y
    elsif @cached_compass_direction == Translations.value('NE') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 7 == 0)
      @world_target_compass_dot_x = ne_compass_dot_x
      @world_target_compass_dot_y = ne_compass_dot_y
    elsif @cached_compass_direction == Translations.value('E') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 6 == 0)
      @world_target_compass_dot_x = e_compass_dot_x
      @world_target_compass_dot_y = e_compass_dot_y
    elsif @cached_compass_direction == Translations.value('SE') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 5 == 0)
      @world_target_compass_dot_x = se_compass_dot_x
      @world_target_compass_dot_y = se_compass_dot_y
    elsif @cached_compass_direction == Translations.value('S') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 4 == 0)
      @world_target_compass_dot_x = s_compass_dot_x
      @world_target_compass_dot_y = s_compass_dot_y
    elsif @cached_compass_direction == Translations.value('SW') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 3 == 0)
      @world_target_compass_dot_x = sw_compass_dot_x
      @world_target_compass_dot_y = sw_compass_dot_y
    elsif @cached_compass_direction == Translations.value('W') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 2 == 0)
      @world_target_compass_dot_x = w_compass_dot_x
      @world_target_compass_dot_y = w_compass_dot_y
    elsif @cached_compass_direction == Translations.value('NW') ||
       (@cached_compass_direction == Translations.value('spins') && game.frame_tick[:frame_tick_game] % 1 == 0)
      @world_target_compass_dot_x = nw_compass_dot_x
      @world_target_compass_dot_y = nw_compass_dot_y
    end

    delta_x = @world_target_compass_dot_x - @world_compass_dot_x
    delta_y = @world_target_compass_dot_y - @world_compass_dot_y
    @world_compass_dot_x += (delta_x * 0.6 * 0.6).round(2)
    @world_compass_dot_y += (delta_y * 0.6 * 0.6).round(2)

    [
      [@world_compass_dot_x.round.to_i,
       @world_compass_dot_y.round.to_i, 5, 5, *red]
    ]
  end

  def world_tick game
    @world_show_scene ||= 15
    if @world_show_scene_method
      @world_show_scene -= 1
      if @world_show_scene <= 0
        self.send(@world_show_scene_method, game)
        @world_show_scene_method = nil
        @world_show_scene = 15
        @builder_button_slots = {}
        @outside_label_spots = {}
        @outside_button_spots = {}
        game.tick
      end
    end
  end

  def world_shown game
    queue_music 'sounds/world.ogg', true
    @previous_compass_direction = game.world.compass_direction
    @supply_flashes.clear
    @supply_flashes_error.clear
    @flashes.clear
    @world_disable_controls = false
    @world_current_fire_state_for_world = game.room.fire
    world_invalidate_cache game
    update_cache game, :force
  end

  def world_invalidate_cache game
    @world_aesthetics_label_cache ||= []
    @world_aesthetics_label_cache.clear
    @world_landmark_labels_cache ||= []
    @world_landmark_labels_cache.clear
  end

  def unload_non_embark_items game
    loot = []

    game.world.stores.keys.each do |key|
      game.stores[key] ||= 0 #set it to zero so that it is available/"seen" from merchants

      next if game.world.embark_items.include? key

      loot << "#{key}: #{game.world.stores[key]}"

      game.world.unload_stores key, game.stores
    end

    if loot.count > 0
      if game.language == :japanese
        game.history << "a safe return."
      else
        game.history << "a safe return. spoils from journey: #{loot.join(", ")}."
      end
    else
      game.history << "a safe return."
    end
  end

  def show_embark_scene game
    show_scene :embark, :embark_progress_bar, game
  end

  def show_world_event_scene game
    show_scene :world_event, nil, game
  end

  def layout_determine_world_event game
    if @previous_compass_direction != game.world.compass_direction
      x, y, _ = world_compass_label game
      queue_flash :compass, x - 5, y - 30, 235, 35, 200, 50, 50, 1 / 100.0
      @previous_compass_direction = game.world.compass_direction
    end


    @previous_fight_risk ||= game.world.enemy_presence_symbol

    if @previous_fight_risk != game.world.enemy_presence_symbol &&
       game.world.enemy_presence_symbol == :very_very_high &&
       game.world.aesthetics[[game.world.x, game.world.y]] != :road &&
       !game.world.on_village? &&
       !game.world.has_event?
      if rand < 0.1
        queue_sound 'sounds/scare.wav'
      end
    end

    @previous_fight_risk = game.world.enemy_presence_symbol

    if game.world.dead?
      @world_disable_controls = true
      @world_show_scene_method = :show_embark_scene

      case game.world.death_reason
      when :starvation
        game.history << "the world fades, collapsed from starvation."
      when :dehydration
        game.history << "the world fades, collapsed from dehydration."
      when :battle
        game.history << "the world fades, defeated in battle."
      end

      return
    end

    if game.world.on_village?
      @world_disable_controls = true
      unload_non_embark_items game
      @world_show_scene_method = :show_embark_scene
      @supply_flashes.clear
      @supply_flashes_error.clear
      @flashes.clear
      return
    end

    if game.world.has_event?
      @world_disable_controls = true
      @world_show_scene_method = :show_world_event_scene
      return
    end
  end

  def update_cache game, intent
    if intent == :force
      game.world.height.times do |i|
        @world_aesthetics_label_cache[i] = nil
        @world_landmark_labels_cache[i] = nil
      end
    else
      game.world.last_revealed_rows.each do |y|
        @world_aesthetics_label_cache[y] = nil
        @world_landmark_labels_cache[y] = nil
      end
    end

    world_aesthetics game
    world_landmarks game
  end

  def layout_move_world_left game
    return unless game.active_module == :world
    return if @world_disable_controls
    @cached_compass_direction = nil
    @parent_delegate.cached_labels = nil
    game.world.move_left
    update_cache game, :left
    @parent_delegate.cached_labels = nil
    layout_determine_world_event game
  end

  def layout_move_world_right game
    return unless game.active_module == :world
    return if @world_disable_controls
    @cached_compass_direction = nil
    @parent_delegate.cached_labels = nil
    game.world.move_right
    update_cache game, :right
    @parent_delegate.cached_labels = nil
    layout_determine_world_event game
  end

  def layout_move_world_down game, player
    return unless game.active_module == :world
    return if @world_disable_controls
    @cached_compass_direction = nil
    @parent_delegate.cached_labels = nil
    game.world.move_down
    update_cache game, :down
    @parent_delegate.cached_labels = nil
    layout_determine_world_event game
  end

  def layout_move_world_up game, player
    return unless game.active_module == :world
    return if @world_disable_controls
    @cached_compass_direction = nil
    @parent_delegate.cached_labels = nil
    game.world.move_up
    update_cache game, :up
    @parent_delegate.cached_labels = nil
    layout_determine_world_event game
  end

  def world_labels game
    return nil unless game.active_module == :world

    [world_water_label(game),
     world_food_label(game),
     world_compass_label(game),
     world_locket_label(game)].reject { |l| l.nil? }
  end

  def world_sprites game
    return nil unless game.active_module == :world

    [*world_landmarks_label(game),
     *world_aesthetics_label(game)
    ].reject { |l| l.nil? }
  end

  def world_line_height
    16
  end

  def landmark_map
    {
      :village => "A", :outpost => "P", :iron_mine => "I", :iron_mine_cleared => "I",
      :coal_mine => "C", :coal_mine_cleared => "C", :sulphur_mine => "S",
      :sulphur_mine_cleared => "S", :house => "H", :close_house => "H",
      :cave => "V", :town => "O", :city => "Y", :ship => "W",
      :borehole => "B", :battlefield => "F", :swamp => "M"
    }
  end

  def aesthetic_map
    {
      :forest => "+",
      :field => "|",
      :barrens => ".",
      :road => "#"
    }
  end

  def world_landmarks_for game, row
    str = ""
    return nil if @world_landmark_labels_cache[row]
    game.world.width.times do |i|
      landmark = game.world.landmarks[[i, row]]
      visible = game.world[[i, row]]
      cleared = game.world.cleared[[i, row]]
      if landmark && visible == :visible && !cleared
        str.concat "#{landmark_map[landmark]}"
      elsif visible == :you
        str.concat "@"
      else
        str.concat " "
      end
    end
    str
  end

  def world_landmarks game
    game.world
        .height.times
        .map { |i| world_landmarks_for game, i }
        .each_with_index.map do |l, i|
          if l && !@world_landmark_labels_cache[i]
              @world_landmark_labels_cache[i] =
                [40,
                 70 + world_line_height * (i - 1),
                 l,
                 font_sm_sm,
                 left_aligned]
          end
        end
  end

  def world_aesthetics_for game, row
    str = ""
    return nil if @world_aesthetics_label_cache[row]
    game.world.width.times do |i|
      aesthetic = game.world.aesthetics[[i, row]]
      has_landmark = game.world.landmark_at?(i, row)
      visible = game.world[[i, row]] == :visible
      cleared = game.world.cleared[[i, row]]
      if aesthetic && !has_landmark && visible
        str.concat "#{aesthetic_map[aesthetic]}"
      elsif has_landmark && cleared && game.world[[i, row]] != :you
        str.concat "#{landmark_map[game.world.landmarks[[i, row]]]}"
      else
        str.concat " "
      end
    end
    str
  end

  def world_aesthetics game
    square = 40
    aesthetics_text_color =
      if game.room.fire_is_out?
        dark_gray
      elsif game.room.fire == :burning
        gray
      elsif game.room.fire == :smoldering || game.room.fire == :flickering
        dark_gray
      else
        really_light_gray
      end

    game.world
        .height.times
        .map { |i| world_aesthetics_for game, i }
        .each_with_index.map do |l, i|
          if l
            @world_aesthetics_label_cache[i] =
              [40,
               70 + world_line_height * (i - 1),
               l,
               font_sm_sm,
               left_aligned, *aesthetics_text_color]
          end
        end
  end

  def world_landmarks_label game
    if @world_current_fire_state_for_world != game.room.fire
      @world_current_fire_state_for_world = game.room.fire
      update_cache game, :force
      queue_sound 'sounds/voices.wav'
    end
    @world_landmark_labels_cache
  end

  def world_aesthetics_label game
    if @world_current_fire_state_for_world != game.room.fire
      @world_current_fire_state_for_world = game.room.fire
      update_cache game, :force
    end
    @world_aesthetics_label_cache
  end

  def world_water_label game
    if (game.stores[:her_locket] || 0) == 0
      [300,
       30,
       "#{Translations.value("water")}: #{game.world.water.round}", 1, left_aligned]
    else
      [130,
       30,
       "#{Translations.value("water")}: #{game.world.water.round}", 1, left_aligned]
    end
  end

  def world_food_label game
    if (game.stores[:her_locket] || 0) == 0
      [880,
       30,
       "#{Translations.value("food")}: #{(game.world.stores[:cured_meat] || 0).round}", 1, left_aligned]
    else
      [730,
       30,
       "#{Translations.value("food")}: #{(game.world.stores[:cured_meat] || 0).round}", 1, left_aligned]
    end
  end

  def world_compass_label game
    if (game.stores[:her_locket] || 0) == 0
      [560,
       30,
       Translations.value("compass: #{game.world.compass_direction}"), 1, left_aligned]
    else
      [420,
       30,
       Translations.value("compass: #{game.world.compass_direction}"), 1, left_aligned]
    end
  end

  def world_locket_label game
    if (game.stores[:her_locket] || 0) == 0

    else
      [960,
       30,
       Translations.value("locket: ") + Translations.value("#{game.world.locket_glow}"), 1, left_aligned]
    end
  end
end
