module LayoutWorldEvent
  def world_event_nav
    [:world_event_option_1_button,
     :world_event_option_2_button,
     :world_event_option_3_button]
  end

  def world_event_dismissed game
  end

  def world_event_shown game
    @supply_flashes.clear
    @supply_flashes_error.clear
    @flashes.clear
  end

  def world_event_labels game
    return nil unless game.active_module == :world_event

    if @world_current_fire_state_for_world != game.room.fire
      @world_current_fire_state_for_world = game.room.fire
      queue_sound 'sounds/voices.wav'
    end

    [world_event_title(game),
     *world_event_text(game)].reject { |l| l.nil? }
  end

  def world_event_buttons game
    return nil unless game.active_module == :world_event

    [
      world_event_option_1_button(game),
      world_event_option_2_button(game),
      world_event_option_3_button(game)
    ]
  end

  def world_event_select_option index, game
    if world_event_just_loot? game
      show_scene :loot, nil, game
    else
      current_scene = world_event_current_scene game
      key = current_scene[:options].keys[index]
      result = game.world.event.change_scene(key)

      if result == false
        alert_show "", ["don't have the supplies."], :world_event, game
        return
      end

      if !game.world.event
        show_scene :world, nil, game
      elsif game.world.event && game.world.event.in_battle?
        show_scene :world_event_battle, :world_event_battle_fists_progress_bar, game
      elsif game.world.event && game.world.event.current_scene == :end
        game.world.event = nil
        show_scene :world, nil, game
      end
    end
  end

  def world_event_accept_pressed game, player
    return unless game.active_module == :world_event

    if @selected_control[player] == :world_event_option_1_button
      world_event_select_option 0, game
      return true
    end

    if @selected_control[player] == :world_event_option_2_button
      world_event_select_option 1, game

      return true
    end
  end

  def world_event_option_button_text game, index
    current_scene = world_event_current_scene game
    key = current_scene[:options].keys[index]
    cost = " (#{game.world.event.cost(key)})"
    cost = "" if cost == " ()"
    "#{Translations.value(key)}#{Translations.value(cost)}"
  end

  def world_event_current_scene game
    return { text: [] } if !game.world.event

    game.world.event.scenes[game.world.event.current_scene]
  end

  def world_event_in_battle? game
    game.world.event && game.world.event.in_battle?
  end

  def world_event_just_loot? game
    current_scene = world_event_current_scene(game)
    !world_event_in_battle?(game) && current_scene[:loot]
  end

  def world_event_option_1_button game
    return nil unless game.world.event
    return nil unless world_event_current_scene(game)[:options].keys[0]
    if world_event_just_loot? game
      [340, (height / 2 ).to_i, 560, 30, Translations.value("loot...")]
    else
      [340, (height / 2 ).to_i, 560, 30, world_event_option_button_text(game, 0)]
    end
  end

  def world_event_option_2_button game
    return nil unless game.world.event
    return nil unless world_event_current_scene(game)[:options].keys[1]
    return nil if world_event_just_loot? game

    [340, (height / 2 ).to_i + 40,
     560, 30, world_event_option_button_text(game, 1)]
  end

  def world_event_option_3_button game
    return nil unless game.world.event
    return nil unless world_event_current_scene(game)[:options].keys[2]
    return nil if world_event_just_loot? game

    [340, (height / 2 ).to_i + 77,
     560, 30, world_event_option_button_text(game, 2)]
  end

  def world_event_text game
    return nil unless game.world.event

    text_with_location = []
    starting_x = 340

    block_height = 0

    starting_y = 75
    max_height = (height / 2) - 75

    world_event_current_scene(game)[:text]
      .map { |t| long_lines_split Translations.value(t), 41 }
      .each do |t|
      t.each do |line|
        text_with_location << [starting_x, block_height, line, 0]
        block_height += 23
      end
      block_height += 15
    end

    offset = 0
    if block_height < (max_height - starting_y)
      offset = (max_height - starting_y) - (block_height)
      offset = offset / 2
    end

    text_with_location = text_with_location.map do |t|
      [t[0], (t[1] + offset).to_i + starting_y, t[2], t[3]]
    end

    text_with_location
  end

  def world_event_title game
    return nil unless game.world.event

    square = 40

    [(width / 2).to_i,
     square - 10,
     Translations.value(game.world.event.title), 1, 1]
  end
end
