module LayoutWorldEventBattle
  def world_event_battle_nav
    [:world_event_battle_dead_button,
     :world_event_battle_loot_button,
     :world_event_battle_fists_progress_bar,
     :world_event_battle_bone_spear_progress_bar,
     :world_event_battle_iron_sword_progress_bar,
     :world_event_battle_steel_sword_progress_bar,
     :world_event_battle_katana_progress_bar,
     :world_event_battle_rifle_progress_bar,
     :world_event_battle_laser_progress_bar,
     :world_event_battle_grenade_progress_bar,
     :world_event_battle_bolas_progress_bar,
     :world_event_battle_cured_meat_progress_bar]
  end

  def world_event_battle_dismissed game

  end

  def world_event_battle_shown game
    @world_event_spots = {}
    @world_event_previous_battle_tick_result = nil
    @world_event_battle_last_attack_result = nil
    @world_event_end_battle_debounce = 60
    @contraption_current_scene = :battle if !contraption_hidden? game
    game.world.init_attacks
  end

  def world_event_play_attack_sound id
    @attack_sounds ||= {
      punch: 'sounds/punch.wav',
      stab: 'sounds/stab.wav',
      swing: 'sounds/swing.wav',
      slash: 'sounds/slash.wav',
      draw: 'sounds/draw.wav',
      shoot: 'sounds/shoot.wav',
      blast: 'sounds/blast.wav',
      stun: 'sounds/stun.wav',
      lob: 'sounds/lob.wav',
    }

    queue_sound @attack_sounds[id] if @attack_sounds[id]
  end

  def world_event_battle_attack_with id, game
    return if game.world.dead?
    return true if game.world.event.battle_complete?
    attack_type = game.world.attacks[id]
    return if !attack_type.can_use?
    @world_event_battle_last_attack_result = game.world.event.attack(attack_type)
    if @world_event_battle_last_attack_result.missed
      queue_sound 'sounds/missed.wav'
    else
      world_event_play_attack_sound id
    end
    if attack_type.melee?
      world_event_animate_melee_player!
    else
      world_event_animate_bullet_player!
    end
    @parent_delegate.invalidate_cache game, true
  end

  def world_event_battle_accept_pressed game, player
    return true if !game.world.event

    if @selected_control[player] == :world_event_battle_cured_meat_progress_bar
      return true if game.world.dead?
      if game.world.can_eat_meat?
        queue_sound 'sounds/eat.wav'
        game.world.eat_meat
        @world_event_player_healed = true
        @world_event_battle_last_enemy_attack_result_show_ticks = 60
      end

      return true
    end

    if @selected_control[player] == :world_event_battle_dead_button
      game.history << "the world fades, defeated in battle."
      show_scene :embark, nil, game
      return true
    end

    if @selected_control[player] == :world_event_battle_fists_progress_bar
      world_event_battle_attack_with :punch, game
      return true
    end

    if @selected_control[player] == :world_event_battle_bone_spear_progress_bar
      world_event_battle_attack_with :stab, game
      return true
    end

    if @selected_control[player] == :world_event_battle_iron_sword_progress_bar
      world_event_battle_attack_with :swing, game
      return true
    end

    if @selected_control[player] == :world_event_battle_steel_sword_progress_bar
      world_event_battle_attack_with :slash, game
      return true
    end

    if @selected_control[player] == :world_event_battle_katana_progress_bar
      world_event_battle_attack_with :draw, game
      return true
    end

    if @selected_control[player] == :world_event_battle_rifle_progress_bar
      world_event_battle_attack_with :shoot, game
      return true
    end

    if @selected_control[player] == :world_event_battle_laser_progress_bar
      world_event_battle_attack_with :blast, game
      return true
    end

    if @selected_control[player] == :world_event_battle_grenade_progress_bar
      world_event_battle_attack_with :lob, game
      return true
    end

    if @selected_control[player] == :world_event_battle_bolas_progress_bar
      world_event_battle_attack_with :stun, game
      return true
    end

    if @selected_control[player] == :world_event_battle_loot_button
      show_scene :loot, nil, game
      return true
    end

    return false
  end

  def world_event_battle_tick game
    return unless game.active_module == :world_event_battle
    return unless world_event_battle_in_battle? game

    if @world_event_previous_battle_tick_result != game.last_battle_tick_result
      if (game.last_battle_tick_result.attacked)
        if world_event_battle_ranged_enemies.include?(world_event_battle_current_scene(game)[:enemy])
          if game.last_battle_tick_result.missed
            queue_sound 'sounds/missed.wav'
          else
            queue_sound 'sounds/enemy_ranged.wav'
          end
          world_event_animate_bullet_enemy!
        else
          if game.last_battle_tick_result.missed
            queue_sound 'sounds/missed.wav'
          else
            queue_sound 'sounds/enemy_melee.wav'
          end
          world_event_animate_melee_enemy!
        end
      end
      @world_event_previous_battle_tick_result = game.last_battle_tick_result
    end
  end

  def world_event_battle_in_battle? game
    return false unless game.world.event
    return false unless game.world.event.in_battle?
    return true
  end

  def world_event_battle_labels game
    return nil unless game.active_module == :world_event_battle

    if @world_current_fire_state_for_world != game.room.fire
      @world_current_fire_state_for_world = game.room.fire
      queue_sound 'sounds/voices.wav'
    end

    player_label = world_event_battle_player_label(game).shift
    enemy_label = world_event_battle_enemy_label(game).shift

    [world_event_battle_title(game),
     *world_event_battle_text(game),
     player_label,
     enemy_label
    ].reject { |l| l.nil? }
  end

  def world_event_battle_sprites game
    return nil unless game.active_module == :world_event_battle

    player_label = world_event_battle_player_label(game)
    player_label.shift
    enemy_label = world_event_battle_enemy_label(game)
    enemy_label.shift

    [
      *player_label,
      world_event_battle_player_bullet_label(game),
      *enemy_label,
      world_event_battle_enemy_bullet_label(game)
    ].reject { |l| l.nil? }
  end

  def world_event_battle_dead_button(game)
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_in_battle? game
    return nil unless game.world.dead?
    @world_event_end_battle_debounce -= 1
    return nil if @world_event_end_battle_debounce > 0

    [380, 320, 520, 30, Translations.value("death...")]
  end

  def world_event_battle_loot_button(game)
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_in_battle? game
    return nil if game.world.dead?
    return nil unless game.world.event.battle_complete?
    @world_event_end_battle_debounce -= 1
    return nil if @world_event_end_battle_debounce > 0

    [380, 320, 520, 30, Translations.value("loot...")]
  end

  def world_event_battle_buttons game
    [
      world_event_battle_dead_button(game),
      world_event_battle_loot_button(game)
    ].reject { |b| b.nil? }
  end

  def world_event_battle_current_scene game
    return { text: [] } if !game.world.event

    game.world.event.scenes[game.world.event.current_scene]
  end

  def world_event_battle_enemy_char_map
    {
      :beast => "B", :cave_lizard => "L", :lizard => "L",
      :snarling_beast => "B", :strange_bird => "B", :man_eater => "E",
      :gaunt_man => "M", :scavenger => "S", :feral_terror => "F",
      :soldier => "D", :sniper => "H", :thug => "T",
      :vigilante => "V", :frail_man => "M", :bird => "B",
      :rats => "RRR", :veteran => "V", :commando => "C",
      :squatter => "S", :youth => "Y", :man => "M",
      :chief => "C", :beastly_matriarch => "M"
    }
  end

  def world_event_battle_should_show_progress_bar? game, id
    return false unless world_event_battle_in_battle? game
    return false if game.world.dead?
    return false if game.world.event.battle_complete?
    return true if game.world.available_attacks.include? id
    return true if id == :cured_meat
    return true if id == :stun && !game.world.stores[:bolas].nil?
    return true if id == :lob && !game.world.stores[:grenade].nil?
    return false
  end

  def world_event_battle_ranged_enemies
    [
      :soldier,
      :sniper,
      :commando,
    ]
  end

  def world_event_battle_progress_bar_y_px game, id
    if !@world_event_spots[id]
      @world_event_spots[id] = 320 + (@world_event_spots.keys.length * 37)
    end

    @world_event_spots[id]
  end

  def world_event_battle_fists_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :punch
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 0),
                        w:    520,
                        h:    30,
                        text: Translations.value("fists"),
                        perc: game.world.attacks[:punch].percentage,
                        rate: nil
  end

  def world_event_battle_bone_spear_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :stab
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 1),
                        w:    520,
                        h:    30,
                        text: Translations.value("bone spear"),
                        perc: game.world.attacks[:stab].percentage,
                        rate: nil
  end

  def world_event_battle_iron_sword_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :swing
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 2),
                        w:    520,
                        h:    30,
                        text: Translations.value("iron sword"),
                        perc: game.world.attacks[:swing].percentage,
                        rate: nil
  end

  def world_event_battle_steel_sword_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :slash
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 3),
                        w:    520,
                        h:    30,
                        text: Translations.value("steel sword"),
                        perc: game.world.attacks[:slash].percentage,
                        rate: nil
  end

  def world_event_battle_katana_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :draw
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 4),
                        w:    520,
                        h:    30,
                        text: Translations.value("katana"),
                        perc: game.world.attacks[:draw].percentage,
                        rate: nil
  end

  def world_event_battle_rifle_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :shoot
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 5),
                        w:    520,
                        h:    30,
                        text: Translations.value("rifle (bullets: ") + "#{game.world.stores[:bullets] || 0})",
                        perc: game.world.attacks[:shoot].percentage,
                        rate: nil
  end

  def world_event_battle_laser_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :blast
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 6),
                        w:    520,
                        h:    30,
                        text: Translations.value("laser (battery: ") + "#{game.world.stores[:battery] || 0})",
                        perc: game.world.attacks[:blast].percentage,
                        rate: nil
  end

  def world_event_battle_grenade_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :lob
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 7),
                        w:    520,
                        h:    30,
                        text: Translations.value("grenade (count: ") + "#{game.world.stores[:grenade]})",
                        perc: game.world.attacks[:lob].percentage,
                        rate: nil
  end

  def world_event_battle_bolas_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :stun
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 8),
                        w:    520,
                        h:    30,
                        text: Translations.value("bolas (count: ") + "#{game.world.stores[:bolas]})",
                        perc: game.world.attacks[:stun].percentage,
                        rate: nil
  end

  def world_event_battle_cured_meat_progress_bar game
    return nil unless game.active_module == :world_event_battle
    return nil unless world_event_battle_should_show_progress_bar? game, :cured_meat
    create_progress_bar x:    380,
                        y:    world_event_battle_progress_bar_y_px(game, 9),
                        w:    520,
                        h:    30,
                        text: Translations.value("cured meat (+#{game.world.meat_heal} hp:") + " #{game.world.stores[:cured_meat]})",
                        perc: game.world.eat_meat_precentage,
                        rate: nil
  end


  def world_event_battle_progress_bars game
    return nil unless game.active_module == :world_event_battle
    return nil unless game.world.event
    return nil unless game.world.event.in_battle?

    [
      world_event_battle_fists_progress_bar(game),
      world_event_battle_bone_spear_progress_bar(game),
      world_event_battle_iron_sword_progress_bar(game),
      world_event_battle_steel_sword_progress_bar(game),
      world_event_battle_katana_progress_bar(game),
      world_event_battle_rifle_progress_bar(game),
      world_event_battle_laser_progress_bar(game),
      world_event_battle_grenade_progress_bar(game),
      world_event_battle_bolas_progress_bar(game),
      world_event_battle_cured_meat_progress_bar(game)
    ].reject { |r| r.nil? }
  end

  def world_event_battle_enemy_bullet_label game
    return nil unless game.active_module == :world_event_battle
    return nil unless game.world.event
    return nil unless game.world.event.in_battle?
    return nil unless @world_event_animate_bullet_enemy

    @world_event_animate_bullet_enemy_x ||= 820
    diff = (@world_event_animate_bullet_enemy_x - 440).abs
    @world_event_animate_bullet_enemy_x -= diff * 0.30
    @world_event_animate_bullet_enemy_x = @world_event_animate_bullet_enemy_x.to_i

    if diff < 40
      @world_event_animate_bullet_enemy = false
    end

    [
      @world_event_animate_bullet_enemy_x + 20,
      240,
      "O",
      font_med,
      left_aligned
    ]
  end

  def world_event_battle_enemy_label game
    return nil unless game.active_module == :world_event_battle
    return nil unless game.world.event
    return nil unless game.world.event.in_battle?

    if @world_event_animate_melee_enemy
      @world_event_animate_melee_enemy_x ||= 820
      @world_event_animate_melee_enemy_direction ||= -1
      if @world_event_animate_melee_enemy_direction == 1
        diff = (@world_event_animate_melee_enemy_x - 820).abs
      else
        diff = (@world_event_animate_melee_enemy_x - 440).abs
      end

      @world_event_animate_melee_enemy_x += diff * 0.35 * @world_event_animate_melee_enemy_direction
      @world_event_animate_melee_enemy_x = @world_event_animate_melee_enemy_x.to_i

      if diff < 40 && @world_event_animate_melee_enemy_direction == 1
        @world_event_animate_melee_enemy = false
      elsif diff < 40
        @world_event_animate_melee_enemy_direction = 1
      end
    else
      @world_event_animate_melee_enemy_x = 820
    end

    current_scene = world_event_battle_current_scene(game)
    char = current_scene[:enemy]

    @world_event_battle_last_attack_result_show_ticks ||= 0

    enemy_damage = ""
    if @world_event_battle_last_attack_result
      if @world_event_battle_last_attack_result.attacked
        @world_event_battle_last_attack_result_show_ticks = 60 if @world_event_battle_last_attack_result_show_ticks <= 0
        if @world_event_battle_last_attack_result.missed
          enemy_damage = "miss"
        elsif @world_event_battle_last_attack_result.weapon == :bolas
          enemy_damage = "stun"
        else
          enemy_damage = "-#{@world_event_battle_last_attack_result.damage.to_s}"
        end
      end
    end

    @world_event_battle_last_attack_result_show_ticks -= 1 if @world_event_battle_last_attack_result_show_ticks > 0

    if @world_event_battle_last_attack_result_show_ticks <= 0
      enemy_damage = ""
      @world_event_battle_last_attack_result = nil
    end

    if enemy_damage == "" || enemy_damage == "miss" || enemy_damage == "stun"
      enemy_damage_label = [
        @world_event_animate_melee_enemy_x - 3,
        200,
        Translations.value(enemy_damage),
        font_med,
        center_aligned
      ]
    else
      enemy_damage_label = [
        @world_event_animate_melee_enemy_x - 3,
        200,
        enemy_damage,
        font_med,
        center_aligned,
        167, 31, 25
      ]
    end

    [
      enemy_damage_label,
      [
        @world_event_animate_melee_enemy_x - 3,
        260,
        world_event_battle_enemy_char_map[char],
        font_lg_lg,
        center_aligned
      ],
      [
        @world_event_animate_melee_enemy_x + 80,
        300,
        "#{current_scene[:health]} / #{current_scene[:max_health]} hp",
        font_med,
        2
      ]
    ]
  end

  def world_event_animate_melee_enemy!
    return if @world_event_animate_melee_enemy
    @world_event_animate_melee_enemy = true
    @world_event_animate_melee_enemy_x = nil
    @world_event_animate_melee_enemy_direction = nil
  end

  def world_event_animate_bullet_enemy!
    return if @world_event_animate_bullet_enemy
    @world_event_animate_bullet_enemy = true
    @world_event_animate_bullet_enemy_x = nil
  end

  def world_event_animate_melee_player!
    return if @world_event_animate_melee_player
    @world_event_animate_melee_player = true
    @world_event_animate_melee_player_x = nil
    @world_event_animate_melee_player_direction = nil
  end

  def world_event_animate_bullet_player!
    return if @world_event_animate_bullet_player
    @world_event_animate_bullet_player = true
    @world_event_animate_bullet_player_x = nil
  end

  def world_event_battle_player_bullet_label game
    return nil unless game.active_module == :world_event_battle
    return nil unless game.world.event
    return nil unless game.world.event.in_battle?
    return nil unless @world_event_animate_bullet_player

    @world_event_animate_bullet_player_x ||= 440
    diff = (@world_event_animate_bullet_player_x - 820).abs
    @world_event_animate_bullet_player_x += diff * 0.30
    @world_event_animate_bullet_player_x = @world_event_animate_bullet_player_x.to_i

    if diff < 40
      @world_event_animate_bullet_player = false
    end

    [
      @world_event_animate_bullet_player_x - 20,
      240,
      "O",
      font_med,
      left_aligned
    ]
  end

  def world_event_battle_player_label game
    return nil unless game.active_module == :world_event_battle
    return nil unless game.world.event
    return nil unless game.world.event.in_battle?

    if @world_event_animate_melee_player
      @world_event_animate_melee_player_x ||= 440
      @world_event_animate_melee_player_direction ||= 1
      if @world_event_animate_melee_player_direction == 1
        diff = (@world_event_animate_melee_player_x - 820).abs
      else
        diff = (@world_event_animate_melee_player_x - 440).abs
      end

      @world_event_animate_melee_player_x += diff * 0.35 * @world_event_animate_melee_player_direction
      @world_event_animate_melee_player_x = @world_event_animate_melee_player_x.to_i

      if diff < 40 && @world_event_animate_melee_player_direction == -1
        @world_event_animate_melee_player = false
      elsif diff < 40
        @world_event_animate_melee_player_direction = -1
      end
    else
      @world_event_animate_melee_player_x = 440
    end

    @world_event_battle_last_enemy_attack_result_show_ticks ||= 0

    player_damage = ""
    if @world_event_previous_battle_tick_result
      if @world_event_previous_battle_tick_result.attacked
        @world_event_battle_last_enemy_attack_result_show_ticks = 60 if @world_event_battle_last_enemy_attack_result_show_ticks <= 0
        if @world_event_previous_battle_tick_result.missed
          player_damage = "miss"
        else
          player_damage = "-#{@world_event_previous_battle_tick_result.damage.to_s}"
        end
      end
    end

    if @world_event_player_healed
      @world_event_battle_last_enemy_attack_result_show_ticks = 60 if @world_event_battle_last_enemy_attack_result_show_ticks <= 0
      player_damage = "+#{game.world.meat_heal}"
    end

    @world_event_battle_last_enemy_attack_result_show_ticks -= 1 if @world_event_battle_last_enemy_attack_result_show_ticks > 0

    if @world_event_battle_last_enemy_attack_result_show_ticks <= 0
      player_damage = ""
      @world_event_previous_battle_tick_result = nil
      @world_event_player_healed = nil
      game.last_battle_tick_result = nil
    end

    if player_damage == "" || player_damage == "miss"
      player_damage_label = [
        @world_event_animate_melee_player_x - 3,
        200,
        Translations.value(player_damage),
        font_med,
        center_aligned
      ]
    elsif player_damage == "+#{game.world.meat_heal}"
      player_damage_label = [
        @world_event_animate_melee_player_x - 3,
        200,
        player_damage,
        font_med,
        center_aligned,
        *green
      ]
    else
      player_damage_label = [
        @world_event_animate_melee_player_x - 3,
        200,
        player_damage,
        font_med,
        center_aligned,
        *red
      ]
    end

    [
      player_damage_label,
      [
        @world_event_animate_melee_player_x,
        260,
        "@",
        font_lg_lg,
        center_aligned
      ],
      [
        @world_event_animate_melee_player_x - 60,
        300,
        "#{game.world.current_hp} / #{game.world.max_hp} hp",
        font_med,
        left_aligned
      ]
    ]
  end

  def world_event_battle_text game
    return nil unless game.active_module == :world_event_battle
    return nil unless game.world.event
    return nil unless game.world.event.in_battle?

    text_with_location = []
    starting_x = 380
    running_y = 75

    world_event_current_scene(game)[:text]
      .map { |t| long_lines_split Translations.value(t), 39 }
      .each do |t|
      t.each do |line|
        text_with_location << [starting_x, running_y, line, 0]
        running_y += 23
      end
      running_y += 15
    end

    text_with_location
  end

  def world_event_battle_title game
    return nil unless game.active_module == :world_event_battle
    return nil unless game.world.event
    return nil unless game.world.event.in_battle?
    square = 40

    [(width / 2).to_i,
     square - 10,
     Translations.value(game.world.event.title), 1, 1]
  end
end
