class Weapon < Hash
  attr_accessor :attack_delay, :current_cooldown, :damage, :weapon, :consumes

  def initialize game, stores, options
    @game = game
    @stores = stores
    @attack_delay = options[:attack_delay]
    @current_cooldown = options[:attack_delay]
    @damage = options[:damage]
    @weapon = options[:weapon]
    @consumes = options[:consumes]
  end

  def percentage
    battle_ticks_til_ready = @attack_delay - @current_cooldown

    attack_delay_in_frames = @game.frame_tick[:frames_between_battle_ticks] * @attack_delay
    battle_ticks_til_ready_in_frames = attack_delay_in_frames - (@game.frame_tick[:frames_between_battle_ticks] * battle_ticks_til_ready)

    frames_elapsed = @game.frame_tick[:frame_tick_battle] - @game.frame_tick[:last_frame_tick_battle]
    interpolated_percent = (((battle_ticks_til_ready_in_frames + frames_elapsed) / attack_delay_in_frames) * 100).round.to_i
    interpolated_percent = 100 if interpolated_percent > 100
    if (@current_cooldown >= @attack_delay)
      return 100
    elsif interpolated_percent > 100
      return 100
    else
      return interpolated_percent
    end
  end

  def melee?
    return false if @consumes

    return false if damage == 0

    true
  end

  def can_use?
    @current_cooldown >= @attack_delay and (melee? or has_consumable?)
  end

  def has_consumable?
    (@stores[@consumes] || 0) > 0
  end

  def used
    @current_cooldown = -1

    return if !@consumes

    return if !@stores[@consumes]

    @stores[@consumes] -= 1
  end

  def tick
    @current_cooldown += 1
    if @current_cooldown > @attack_delay
      @current_cooldown = @attack_delay
    end
  end
end
