class World < Hash
  @@stub_roll_value = nil
  @@override_aesthetics = nil
  @@disable_landmarks = false

  attr_accessor :x, :y, :aesthetics, :width, :height, :terrain_consistancy,
                :terrain_probs, :landmarks, :event, :stores,
                :current_hp, :apons, :attacks, :meat_cool_down,
                :cleared, :fight_move, :fight_delay,
                :water, :death_reason, :half_width, :half_height, :current_embark_ticks,
                :embark_after, :food_deaths, :battle_deaths, :water_deaths, :punch_count,
                :previous_x, :previous_y, :last_revealed_rows

  def initialize game = nil
    @game = game
    @aesthetics = Hash.new
    @landmarks = Hash.new
    @cleared = Hash.new

    reset
  end

  def world_hash
    self.hash
  end

  def set_landmarks! value
    @landmarks.clear
    value.each { |k, v| @landmarks[k] = v }
  end

  def set_aesthetics! value
    @aesthetics.clear
    value.each { |k, v| @aesthetics[k] = v }
  end

  def set_cleared! value
    @cleared.clear
    value.each { |k, v| @cleared[k] = v }
  end

  def world_hash= value
    self.clear
    value.each { |k, v| self[k] = v }
  end

  def world_stores_hash
    @stores.hash
  end

  def world_stores_hash= value
    @stores.hash.clear
    value.each { |k, v| @stores.hash[k] = v }
  end

  def god_mode!
    @god_mode = true
    @current_hp = 9999
  end

  def disable_god_mode!
    @god_mode = false
  end

  def god_mode?
    @god_mode
  end

  def describe_terrain
    closest_terrains = []

    terrains = {
      :forest => "forest",
      :field => "fields",
      :barrens => "barrens",
    }

    terrains.keys.each do |k|
      closest_terrain = closest(k)
      if(closest_terrain.count == 4 and distance_to(closest_terrain.first[:x], closest_terrain.first[:y]) == 1)
        closest_terrains << "#{Translations.value(terrains[k])}:" + Translations.value("to the north, south, east, and west")
      else
        closest_terrain.map do |loc|
          closest_terrains << "#{Translation.value(terrains[k])}: #{desc_location(loc)}"
        end
      end
    end

    closest_terrains
  end

  def describe_roads
    closest(:road).map do |loc|
      Translations.value("roads:") + " #{desc_location(loc)}"
    end
  end

  def describe_unexplored
    result = []
    closest_terrain = []

    to_filter = fog_locations

    #cradinal directions
    #N,NE,E,SE,S,SW,W,NW

    [:N, :S, :E, :W].each do |dir|
      closest = to_filter
                  .find_all { |loc| compass_direction_loc(loc) == dir }
                  .sort_by { |loc| distance_to_loc loc }
                  .first

      closest_terrain << closest if closest
    end

    if closest_terrain.count < 4
      [:NE, :SE, :NW, :SW].each do |dir|
        closest = to_filter
                    .find_all { |loc| compass_direction_loc(loc) == dir  }
                    .sort_by { |loc| distance_to_loc loc }
                    .first

        closest_terrain << closest if closest
      end
    end

    closest_terrain.map do |loc|
      result << Translations.value("unexplored:") + " #{desc_location(loc)}"
    end.first

    result
  end

  def closest name
    to_filter = visible_locations

    smallest_distance = to_filter.find_all { |loc|
      @aesthetics[[loc[:x], loc[:y]]] == name
    }.map { |loc|
      distance_to loc[:x], loc[:y]
    }.sort.first

    if(smallest_distance)
      return to_filter.find_all { |loc|
        @aesthetics[[loc[:x], loc[:y]]] == name and
          distance_to(loc[:x], loc[:y]) == smallest_distance
      }
    else
      return []
    end
  end

  def describe_landmarks
    visible_landmarks.sort_by { |loc|
      if @landmarks[[loc[:x], loc[:y]]] == :village
        -1000
      else
        distance_to(loc[:x], loc[:y])
      end
    }.reject { |loc|
      @cleared[[loc[:x], loc[:y]]]
    }.map { |loc|
      describe_landmark loc
    }
  end

  def describe_landmark loc
    descriptions = {
      :close_house => "house",
      :iron_mine_cleared => "iron mine",
      :sulphur_mine_cleared => "sulphur mine",
      :coal_mine_cleared =>  "coal mine",
      :borehole => "bore hole"
    }

    entry = @landmarks[[loc[:x], loc[:y]]]
    description = descriptions[entry] || entry
    "#{Translations.value(description)}, #{desc_location loc}"
  end

  def desc_location loc
    distance_x = @x - loc[:x]
    distance_y = @y - loc[:y]
    direction_y = Translations.value("north")
    direction_y = Translations.value("south") if distance_y < 0
    direction_x = Translations.value("west")
    direction_x = Translations.value("east") if distance_x < 0
    return Translations.value("current location") if distance_x == 0 and distance_y == 0
    return "#{distance_x.abs} #{clicks distance_x} #{direction_x}" if distance_y == 0
    return "#{distance_y.abs} #{clicks distance_y} #{direction_y}" if distance_x == 0
    return "#{distance_y.abs} #{clicks distance_y} #{direction_y}, #{distance_x.abs} #{clicks distance_x} #{direction_x}"
  end

  def clicks distance
    return Translations.value("click") if distance.abs == 1
    return Translations.value("clicks")
  end

  def cleared?
    fog_locations.count == 0
  end

  def city_visible?
    visible_locations.any? { |loc| @landmarks[[loc[:x], loc[:y]]] == :city }
  end

  def can_embark?
    @current_embark_ticks >= @embark_after
  end

  def tick
    return if !@game.stores[:compass]

    @current_embark_ticks += 1 if @current_embark_ticks < @embark_after
  end

  def vision_radius
    return 50 if @god_mode

    return 4 if @game.has_perk? :scout

    2
  end

  def reset
    @current_hp = max_hp
    @food_deaths = 0
    @water_deaths = 0
    @battle_deaths = 0
    @punch_count = 0
    @embark_after = 90
    @current_embark_ticks = 0
    @meat_cool_down = meat_delay
    @meat_delay = 0
    self.clear
    @aesthetics.clear
    @landmarks.clear
    @cleared.clear
    @revealed_squares_this_session = []
    @cleared_landmarks_this_session = []
    @vision_radius = 50
    @width = 90
    @height = 40
    @half_width = (@width / 2).to_i
    @half_height = (@height / 2).to_i

    @width.times do |x|
      @height.times do |y|
        if x == @half_width and y == @half_height
          self[[x, y]] = :you
          @x = x
          @y = y
        else
          self[[x, y]] = :fog
        end
      end
    end

    init_fight
    init_stores
    init_attacks
    init_terrain_probabilities
    init_landmark_definitions
    generate_aesthetics
    generate_landmarks if !@@disable_landmarks
    reveal_map

    @stores[:cured_meat] ||= 0
  end

  def embark_items
    [
      :cured_meat,
      :bone_spear,
      :torch,
      :iron_sword,
      :steel_sword,
      :rifle,
      :bullets,
      :carbine,
      :battery,
      :grenade,
      :bolas,
      :katana,
      :jewel
    ]
  end

  def init_fight
    @fight_move = 0
    @fight_delay = 3
  end

  def landmark_locations landmark
    found = nil
    cleared_landmark = "#{landmark}_cleared".to_sym
    @landmarks.keys.each do |key|
      if @landmarks[key] == landmark or @landmarks[key] == cleared_landmark
        found = key
      end
    end
    found
  end

  def mine_cleared? mine
    @landmarks[landmark_locations(mine)] == "#{mine}_cleared".to_sym
  end

  def named_event_cleared? event_name
    cleared[landmark_locations(event_name)]
  end

  def locket_glow
    loc = uncleared_boreholes_by_distance.first

    return "dark" if !loc

    distance = distance_to loc[0], loc[1]

    return "dark" if distance > 25

    return "cold glow" if distance > 20

    return "warm glow" if distance > 15

    return "warmer" if distance > 10

    return "brighter" if distance > 5

    return "blinding"
  end

  def directions_from_direction_tool
    return locket_glow if directions_tool == :locket

    compass_direction
  end

  def compass_direction
    if !mine_cleared? :iron_mine
      compass_direction_string landmark_locations(:iron_mine)
    elsif !mine_cleared? :coal_mine
      compass_direction_string landmark_locations(:coal_mine)
    elsif !mine_cleared? :sulphur_mine
      compass_direction_string landmark_locations(:sulphur_mine)
    elsif !named_event_cleared? :ship
      compass_direction_string landmark_locations(:ship)
    elsif !named_event_cleared? :swamp
      compass_direction_string landmark_locations(:swamp)
    else
      "spins"
    end
  end

  def directions_tool
    return :locket if @game.stores[:her_locket]

    :compass
  end

  def compass_direction_string destination
    return "spins" if !destination

    north_south_or_none = ""
    north_south_or_none = "N" if y > destination[1]
    north_south_or_none = "S" if y < destination[1]
    east_west_or_none = ""
    east_west_or_none = "W" if x > destination[0]
    east_west_or_none = "E" if x < destination[0]

    return "spins" if north_south_or_none == "" and east_west_or_none == ""
    return "#{north_south_or_none}#{east_west_or_none}"
  end

  def compass_direction_loc loc
    loc = { :x => loc[0], :y => loc[1] } if loc.is_a? Array

    distance_x = @x - loc[:x]
    distance_y = @y - loc[:y]
    north = y > loc[:y]
    south = y < loc[:y]
    east = x < loc[:x]
    west = x > loc[:x]

    return :N if distance_x == 0 and north
    return :NE if north and east
    return :E if distance_y == 0 and east
    return :SE if south and east
    return :S if distance_x == 0 and south
    return :SW if south and west
    return :W if distance_y == 0 and west
    return :NW if north and west
  end

  def enemy_presence
    return Translations.value("none, while on roads (the # signs)") if @aesthetics[[x,y]] == :road
    return Translations.value("low") if @fight_move == 0
    return Translations.value("medium") if @fight_move <= (@fight_delay / 2)
    return Translations.value("high") if @fight_move <= @fight_delay
    return Translations.value("very high") if @fight_move > @fight_delay
  end

  def enemy_presence_symbol
    return :road if @aesthetics[[x,y]] == :road
    return :low if @fight_move == 0
    return :medium if @fight_move <= (@fight_delay / 2)
    return :high if @fight_move <= @fight_delay
    return :very_very_high if @fight_move + 4 > @fight_delay
    return :very_high if @fight_move > @fight_delay
  end

  def fog_locations
    locations_for :fog
  end

  def visible_locations
    locations_for :visible
  end

  def visible_landmarks
    result = @landmarks.keys.find_all { |k|
      self[[k[0], k[1]]] == :visible
    }.map { |k| { :x => k[0], :y => k[1] } }
  end

  def locations_for type
    self.keys
      .map { |k| { :x => k[0], :y => k[1] } }
      .find_all { |k| self[[k[:x], k[:y]]] == type }
  end

  def reveal_random
    random_x = 10
    random_y = 10

    random_fog = fog_locations.sample

    if random_fog
      random_x = random_fog[:x]
      random_y = random_fog[:y]
    end

    area = 5

    start_x = random_x - area
    start_y = random_y - area

    end_x = random_x + area
    end_y = random_y + area

    (start_x..end_x).each do |x|
      (start_y..end_y).each do |y|
        current_value = self[[x,y]]
        if current_value == :fog and distance_to(x, y, random_x, random_y) <= area
          self[[x,y]] = :visible
        end
      end
    end

    [random_x, random_y]
  end

  def self.stub_roll with_value
    @@stub_roll_value = with_value
  end

  def self.disable_landmarks
    @@disable_landmarks = true
  end

  def self.enable_landmarks
    @@disable_landmarks = false
  end

  def self.override_aesthetics aesthetic
    @@override_aesthetics = aesthetic
  end

  def roll
    rand
  end

  def take_stores store, from
    move_stores store, from, @stores
  end

  def unload_stores store, to
    @stores[store].times do
      drop_stores store, to
    end
  end

  def drop_stores store, to
    move_stores store, @stores, to

    @stores.delete(store) if @stores[store] == 0 and store != :cured_meat
  end

  def move_stores store, from, to
    return if !from[store] || from[store] == 0
    to[store] ||= 0
    from[store] -= 1
    to[store] += 1
  end

  def tick_meat
    return if can_eat_meat?

    @meat_cool_down += 1

    if @meat_cool_down > meat_delay
      @meat_cool_down = meat_delay
    end
  end

  def init_stores
    @stores = Stores.new
    @stores[:cured_meat] = 0
  end

  def self.consumables
    {
      :shoot => :bullets,
      :blast => :battery,
      :lob => :grenade,
      :stun => :bolas
    }
  end

  def init_attacks
    @attacks = {
      :punch => Weapon.new(@game, @stores, attack_delay: 2, damage: 1, weapon: :fists),
      :stab  => Weapon.new(@game, @stores, attack_delay: 2, damage: 2, weapon: :bone_spear),
      :swing => Weapon.new(@game, @stores, attack_delay: 2, damage: 4, weapon: :iron_sword),
      :slash => Weapon.new(@game, @stores, attack_delay: 2, damage: 6, weapon: :steel_sword),
      :draw  => Weapon.new(@game, @stores, attack_delay: 2, damage: 8, weapon: :katana),
      :shoot => Weapon.new(@game, @stores, attack_delay: 2, damage: 8, weapon: :rifle, consumes: World.consumables[:shoot]),
      :blast => Weapon.new(@game, @stores, attack_delay: 2, damage: 11, weapon: :carbine, consumes: World.consumables[:blast]),
      :lob   => Weapon.new(@game, @stores, attack_delay: 5, damage: 15, weapon: :grenade, consumes: World.consumables[:lob]),
      :stun  => Weapon.new(@game, @stores, attack_delay: 15, damage: 0, weapon: :bolas, consumes: World.consumables[:stun]),
    }
  end

  def available_attacks
    result = []

    result << :punch if !has_melee_weapon?

    attacks.keys.each do |action|
      if stores.has? attacks[action].weapon
        result << action
      end
    end

    result
  end

  def eat_meat_precentage
    battle_ticks_til_ready = meat_delay - @meat_cool_down

    attack_delay_in_frames = @game.frame_tick[:frames_between_battle_ticks] * meat_delay
    battle_ticks_til_ready_in_frames = attack_delay_in_frames - (@game.frame_tick[:frames_between_battle_ticks] * battle_ticks_til_ready)

    frames_elapsed = @game.frame_tick[:frame_tick_battle] - @game.frame_tick[:last_frame_tick_battle]
    interpolated_percent = (((battle_ticks_til_ready_in_frames + frames_elapsed) / attack_delay_in_frames) * 100).round.to_i
    interpolated_percent = 100 if interpolated_percent > 100

    if (@meat_cool_down >= meat_delay)
      return 100
    elsif interpolated_percent > 100
      return 100
    else
      return interpolated_percent
    end
  end

  def can_eat_meat?
    @meat_cool_down >= meat_delay and (@stores[:cured_meat] || 0) > 0
  end

  def meat_delay
    5
  end

  def meat_heal
    return 20 if @game.has_perk?(:gastronome)

    10
  end

  def eat_meat
    @current_hp = 9999 if @god_mode

    return if !can_eat_meat?

    @current_hp += meat_heal

    @current_hp = max_hp if @current_hp > max_hp

    @stores[:cured_meat] -= 1

    @meat_cool_down = 0
  end

  def hash
    self
  end

  def has_melee_weapon?
    attacks.keys.any? do |action|
      stores.has? attacks[action].weapon and attacks[action].melee?
    end
  end

  def uncleared_boreholes
    borehole_locations = []
    @landmarks.keys.each do |loc|
      if @landmarks[loc] == :borehole and !@cleared[loc]
        borehole_locations << [loc[0], loc[1]]
      end
    end

    borehole_locations
  end

  def uncleared_boreholes_by_distance
    uncleared_boreholes.sort_by do |loc|
      distance_to loc[0], loc[1]
    end
  end

  def init_landmark_definitions
    magic_radius = 30
    @landmark_definitions = {
      :outpost => { :num => 0, :min_radius => 0, :max_radius => 0 },
      :iron_mine => { :num => 1, :min_radius => 5, :max_radius => 5 },
      :coal_mine => { :num => 1, :min_radius => 10, :max_radius => 10 },
      :sulphur_mine => { :num => 1, :min_radius => 20, :max_radius => 20 },
      :house => { :num => 20, :min_radius => 3, :max_radius => magic_radius * 1.5 },
      :close_house => { :num => 2, :min_radius => 2, :max_radius => 3 },
      :cave => { :num => 5, :min_radius => 3, :max_radius => 10 },
      :town => { :num => 15, :min_radius => 10, :max_radius => 20 },
      :city => { :num => 25, :min_radius => 20, :max_radius => magic_radius * 1.5 },
      :ship => {:num => 1, :min_radius => 28, :max_radius => 28 },
      :borehole => {:num => 10, :min_radius => 15, :max_radius => magic_radius * 1.5 },
      :battlefield => {:num => 10, :min_radius => 18, :max_radius => magic_radius * 1.5 },
      :swamp => {:num => 1, :min_radius => 40, :max_radius => magic_radius * 1.5 }
    }
  end

  def clear_landmark x, y
    @cleared_landmarks_this_session << [x,y]
    cleared[[x,y]] = true
  end

  def total_deaths
    @food_deaths + @water_deaths + @battle_deaths
  end

  def process_perks
    @game.add_perk :desert_rat if @water_deaths >= 5
    @game.add_perk :slow_metabolism if @food_deaths >= 5
    @game.add_perk :barbarian if @battle_deaths >= 10 #sniper, soldier and feral terral are very difficult
    @game.add_perk :boxer if @punch_count >= 50
    @game.add_perk :martial_artist if @punch_count >= 150
    @game.add_perk :unarmed_master if @punch_count >= 300
  end

  def cause_death reason
    if reason == :battle
      @battle_deaths += 1
    elsif reason == :dehydration
      @water_deaths += 1
    elsif reason == :starvation
      @food_deaths +=1
    end

    @death_reason = reason
    @current_hp = 0
    @current_embark_ticks = 0

    rollback_revealed_squares
    rollback_cleared_landmarks
    rollback_aesthetics
    rollback_landmarks

    drop_all_stores

    process_perks
  end

  def drop_all_stores
    @stores.hash.clear
    @stores[:cured_meat] ||= 0
  end

  def rollback_cleared_landmarks
    @cleared_landmarks_this_session.each do |square|
      cleared[square] = false
    end
  end

  def rollback_aesthetics
    @aesthetics.clear

    @last_aesthetics_state.keys.each do |key|
      @aesthetics[key] = @last_aesthetics_state[key]
    end
  end

  def rollback_landmarks
    @landmarks.clear

    @last_landmarks.keys.each do |key|
      @landmarks[key] = @last_landmarks[key]
    end
  end

  def rollback_revealed_squares
    @revealed_squares_this_session.each do |square|
      self[square] = :fog
    end

    self.keys.each do |square|
      self[square] = :visible if self[square] == :you
    end

    self[[@half_width, @half_height]] = :you
    @x = @half_width
    @y = @half_height

    reveal_map

    @revealed_squares_this_session = []
  end

  def max_distance_till_starvation
    (ticks_til_hunger * (food - 1)) + @hunger_countdown
  end

  def max_distance_till_dehydration
    (@water - water_consumption).floor
  end

  def distance_from_village
    distance_to @half_width, @half_height
  end

  def in_range_of_village greater_than_x, less_than_x, greater_than_y, less_than_y
    dx = (@half_width  - @x).abs
    dy = (@half_height - @y).abs

    if (dx / @half_width.to_f) > (dy / half_height.to_f)
      (dx >= greater_than_x && dx <= less_than_x)
    else
      (dy >= greater_than_y && dy <= less_than_y)
    end
  end

  def food
    @stores[:cured_meat] || 0
  end

  def decrement_hp amount
    @current_hp -= amount

    cause_death :battle if @current_hp <= 0
  end

  def base_carry_space
    100
  end

  def base_water
    10.0
  end

  def base_hp
    25
  end

  def max_hp
    if @game.has_upgrade? :war_mantle
      return base_hp + 55
    elsif @game.has_upgrade? :steel_armour
      return base_hp + 35
    elsif @game.has_upgrade? :iron_armour
      return base_hp + 15
    elsif @game.has_upgrade? :leather_armour
      return base_hp + 5
    else
      return base_hp
    end
  end

  def armour
    if @game.has_upgrade? :war_mantle
      return :war_mantle
    elsif @game.has_upgrade? :steel_armour
      return :steel_armour
    elsif @game.has_upgrade? :iron_armour
      return :iron_armour
    elsif @game.has_upgrade? :leather_armour
      return :leather_armour
    else
      return :rags
    end
  end

  def carry
    if @game.has_upgrade? :convoy
      return { :name => :convoy, :max => base_carry_space + 700 }
    elsif @game.has_upgrade? :wagon
      return { :name => :wagon, :max => base_carry_space + 400 }
    elsif @game.has_upgrade? :rucksack
      return { :name => :rucksack, :max => base_carry_space + 200 }
    else
      return { :name => :bag, :max => base_carry_space }
    end
  end

  def container
    if @game.has_upgrade? :water_tank
      return { :name => :water_tank, :max => base_water + 50 }
    elsif @game.has_upgrade? :cask
      return { :name => :cask, :max => base_water + 20 }
    elsif @game.has_upgrade? :waterskin
      return { :name => :waterskin, :max => base_water + 10 }
    else
      return { :name => :flask, :max => base_water }
    end
  end

  def stores_weight
    @stores.keys.map do |key|
      (@stores[key] || 0) * weight_for(key)
    end.inject(:+) || 0
  end

  def ticks_til_hunger
    return 4 if @game.has_perk? :slow_metabolism

    2
  end

  def begin_journey
    @current_hp = max_hp
    @fight_move = 0
    @death_reason = nil
    @last_landmarks = @landmarks.clone
    @last_aesthetics_state = @aesthetics.clone
    @event = nil
    reset_hunger_countdown
    reset_water_countdown
    reset_outposts
  end

  def reset_outposts
    @cleared.keys.each do |key|
      @cleared.delete(key) if @landmarks[key] == :outpost
    end
  end

  def reset_hunger_countdown
    @stores[:cured_meat] ||= 0
    @hunger_countdown = ticks_til_hunger
  end

  def reset_water_countdown
    @water = container[:max]
  end

  def eat_or_starve
    return if @god_mode
    return if landmark_at? x, y

    if @hunger_countdown > 0
      @hunger_countdown -= 1
    else
      if @stores[:cured_meat] == 0
        cause_death :starvation
      else
        @stores[:cured_meat] -= 1
        @current_hp = max_hp if @current_hp < max_hp
        reset_hunger_countdown
      end
    end
  end

  def drink_or_dehydrate
    return if @god_mode
    return if landmark_at? x, y

    if @water > 0.0
      @water -= water_consumption
    else
      cause_death :dehydration
    end
  end

  def water_consumption
    return 0.5 if @game.has_perk? :desert_rat

    1.0
  end

  def weight_for key
    return 0 if !key

    weights = {
      :bone_spear => 20,
      :iron_sword => 30,
      :steel_sword => 50,
      :rifle => 50,
      :bullets => 1,
      :battery => 2,
      :carbine => 50,
      :bolas => 5
    }

    weights[key] || 10
  end

  def dead?
    @current_hp <= 0
  end

  def has_event?
    !@event.nil?
  end

  def event
    @event
  end

  def on_village?
    @x == @half_width and @y == @half_height
  end

  def init_terrain_probabilities
    @terrain_probs = { :forest => 0.15, :field => 0.35, :barrens => 0.5 }
    @terrain_consistancy = 0.5
  end

  def landmark_at_loc? loc
    landmark_at? loc[:x], loc[:y]
  end

  def landmark_at? x, y
    !!@landmarks[[x, y]]
  end

  def landmark_near? x, y
    return false unless rand > 0.25

    landmark_at?(x - 1, y - 1) ||
    landmark_at?(x - 0, y - 1) ||
    landmark_at?(x + 1, y - 1) ||

    landmark_at?(x - 1, y + 0) ||
    landmark_at?(x - 0, y + 0) ||
    landmark_at?(x + 1, y + 0) ||

    landmark_at?(x - 1, y + 1) ||
    landmark_at?(x - 0, y + 1) ||
    landmark_at?(x + 1, y + 1) ||

    landmark_at?(x - 2, y - 2) ||
    landmark_at?(x - 0, y - 2) ||
    landmark_at?(x + 2, y - 2) ||

    landmark_at?(x - 2, y + 0) ||
    landmark_at?(x - 0, y + 0) ||
    landmark_at?(x + 2, y + 0) ||

    landmark_at?(x - 2, y + 2) ||
    landmark_at?(x - 0, y + 2) ||
    landmark_at?(x + 2, y + 2) ||

    landmark_at?(x - 2, y - 2) ||
    landmark_at?(x - 1, y - 2) ||
    landmark_at?(x + 2, y - 2) ||

    landmark_at?(x - 2, y + 1) ||
    landmark_at?(x - 1, y + 1) ||
    landmark_at?(x + 2, y + 1) ||

    landmark_at?(x - 2, y + 2) ||
    landmark_at?(x - 1, y + 2) ||
    landmark_at?(x + 2, y + 2)
  end

  def generate_landmark min_radius, max_radius, landmark
    x = @half_width
    y = @half_height
    ratio_of_width_to_height = @half_width / @half_height
    while landmark_at?(x, y) || landmark_near?(x, y)
      r = (roll * (max_radius - min_radius)).floor + min_radius
      x_dist = ((roll * r).floor * 1.15).to_i
      y_dist = r - x_dist

      if (y_dist > (@half_height - 2))
        x_dist, y_dist = [y_dist, x_dist]
      end

      x_dist = -x_dist if(roll < 0.5)
      y_dist = -y_dist if(roll < 0.5)
      x = @half_width + x_dist
      x = 0 if(x < 0)
      x = @half_width * 2 if(x > @half_width * 2)
      y = @half_height + y_dist
      y = 0 if(y < 0)
      y = @half_height * 2 if(y > @half_height * 2)
      x = (@half_width * 2) - 1 if x >= (@half_width * 2)
      y = (@half_height * 2) - 1 if y >= (@half_height * 2)
    end

    @landmarks[[x.to_i, y.to_i]] = landmark
  end

  def ship_cleared?
    named_event_cleared? :ship
  end

  def generate_landmarks
    @landmarks[[@half_width,@half_height]] = :village

    @landmark_definitions.keys.each do |k|
      landmark = @landmark_definitions[k]

      landmark[:num].times do |i|
        generate_landmark landmark[:min_radius], landmark[:max_radius], k
      end
    end

    @landmarks[[@half_width,@half_height]] = nil if @@disable_landmarks
  end

  def generate_aesthetics
    @half_width.times do |r|
      r = r + 1
      (r * 8).times do |t|
        x = nil
        y = nil

        if t < (2 * r)
          x = @half_width - r + t
          y = @half_height - r
        elsif t < (4 * r)
          x = @half_width + r
          y = @half_height - (3 * r) + t
        elsif t < (6 * r)
          x = @half_width + (5 * r) - t
          y = @half_height + r
        else
          x = @half_width - r
          y = @half_height + (7 * r) - t
        end

        x = x.to_i
        y = y.to_i

        @aesthetics[[x,y]] = choose_tile(x, y)
      end
    end
  end

  def draw_road
    x_dist = x - @half_width
    y_dist = y - @half_height

    if x_dist.abs > y_dist.abs
      x_intersect = @half_width
      y_intersect = @half_height + y_dist
    else
      x_intersect = @half_width + x_dist
      y_intersect = @half_height
    end

    if x_dist != 0
      x_dir = x_dist.abs / x_dist
      x_dist.abs.times do |x|
        target_x = @half_width + (x_dir * x)
        target_y = y_intersect
        next if landmark_at? target_x.to_i, target_y.to_i

        @aesthetics[[target_x.to_i, target_y.to_i]] = :road
      end
    end

    if y_dist != 0
      y_dir = y_dist.abs / y_dist
      y_dist.abs.times do |y|
        target_x = x_intersect
        target_y = @half_height + (y_dir * y)
        next if landmark_at? target_x.to_i, target_y.to_i

        @aesthetics[[target_x.to_i, target_y.to_i]] = :road
      end
    end
  end

  def choose_tile x, y
    return @@override_aesthetics if @@override_aesthetics

    adjacent = [
      @aesthetics[[x, y - 1]],
      @aesthetics[[x, y + 1]],
      @aesthetics[[x + 1, y]],
      @aesthetics[[x - 1, y]]
    ]

    return :forest if x == 44 && y == 19
    return :forest if x == 45 && y == 19
    return :forest if x == 46 && y == 19

    return :forest if x == 44 && y == 20
    return :forest if x == 45 && y == 20
    return :forest if x == 46 && y == 20

    return :forest if x == 44 && y == 21
    return :forest if x == 45 && y == 21
    return :forest if x == 46 && y == 21

    chances = { }
    non_sticky = 1
    adjacent.each do |i|
      if !i.nil?
        cur = chances[i] || 0
        chances[i] = cur + @terrain_consistancy
        non_sticky -= @terrain_consistancy
      end
    end

    @terrain_probs.keys.each do |t|
      cur = chances[t] || 0
      cur += @terrain_probs[t] * non_sticky
      chances[t] = cur
    end

    sorted = chances.sort_by { |tile, chance| chance }.each

    c = 0
    dice_roll = roll
    result = nil

    sorted.each do |entry|
      value = entry.last

      c += value

      result = entry.first.to_sym if dice_roll < c and result.nil?
    end

    result || :barrens
  end

  def reveal_map
    @last_revealed_rows ||= []
    @last_revealed_rows.clear
    start_x = @x - vision_radius
    start_y = @y - vision_radius

    end_x = @x + vision_radius
    end_y = @y + vision_radius

    @last_revealed_rows << @previous_y
    @last_revealed_rows << @y

    (start_x..end_x).each do |x|
      (start_y..end_y).each do |y|
        current_value = self[[x,y]]
        if current_value == :fog and distance_to(x, y) <= vision_radius
          @revealed_squares_this_session << [x,y]
          @last_revealed_rows << y
          self[[x,y]] = :visible
        end
      end
    end
  end

  def self.event_map
    {
      :cave => CaveEvent,
      :house => HouseEvent,
      :close_house => SafeHouseEvent,
      :iron_mine => IronMineEvent,
      :iron_mine_cleared => IronMineClearedEvent,
      :coal_mine => CoalMineEvent,
      :coal_mine_cleared => CoalMineClearedEvent,
      :sulphur_mine => SulphurMineEvent,
      :sulphur_mine_cleared => SulphurMineClearedEvent,
      :swamp => SwampEvent,
      :ship => ShipEvent,
      :town => TownEvent,
      :city => CityEvent,
      :battlefield => BattlefieldEvent,
      :borehole => BoreholeEvent,
      :outpost => OutpostEvent
    }
  end

  def teleport_to landmark
    self[[@x, @y]] = :visible
    @x, @y = landmark_locations(landmark)
    self[[@x, @y]] = :you
    commit
    reveal_map
    init_event
  end

  def init_event
    return if event_cleared? x, y
    landmark = @landmarks[[x,y]]

    if landmark
      event_class = World.event_map[landmark]
      if event_class
        @event = event_class.new @game
        @event.init_scenes

        @water = container[:max] if @event.replenishes_water?
      end
    end
  end

  def self.encounters
    # horizontal radius start, horizontal radius stop, vertical radius start, vertical radius stop
    # --- Emacs Calculator Mode ---                                                                                        |* TODO La$
    # 2:  2.25                                                                                                             |* TODO No$
    # 1:  0.444444444444                                                                                                   |* TODO No$

    easy   = [0.00, 0.30]
    medium = [0.30, 0.60]
    hard   = [0.60, 1.00]

    [
      {
        :event => SnarlingBeastEvent,
        :aesthetics => :forest,
        :range => easy
      },
      {
        :event => StrangeBirdEvent,
        :aesthetics => :field,
        :range => easy
      },
      {
        :event => GauntManEvent,
        :aesthetics => :barrens,
        :range => easy
      },
      {
        :event => ManEaterEvent,
        :aesthetics => :forest,
        :range => medium
      },
      {
        :event => HugeLizardEvent,
        :aesthetics => :field,
        :range => medium
      },
      {
        :event => ScavengerEvent,
        :aesthetics => :barrens,
        :range => medium
      },
      {
        :event => FeralTerrorEvent,
        :aesthetics => :forest,
        :range => hard
      },
      {
        :event => SniperEvent,
        :aesthetics => :field,
        :range => hard
      },
      {
        :event => SoldierEvent,
        :aesthetics => :barrens,
        :range => hard
      },
    ]
  end

  def fight_chance
    return 0.10 if @game.has_perk? :stealthy

    return 0.20
  end

  def check_fight
    return if landmark_at? x, y

    return if @aesthetics[[x,y]] == :road

    @fight_move += 1

    if @fight_move > @fight_delay and roll <= fight_chance
      encounter =
        World.encounters
             .select do |h|
               @aesthetics[[x,y]] == h[:aesthetics] and
                 in_range_of_village(h[:range][0] * @half_width,
                                     h[:range][1] * @half_width,
                                     h[:range][0] * @half_height,
                                     h[:range][1] * @half_height)
             end
             .map { |h| h[:event] }
             .first

      @event = encounter.new @game if encounter

      @fight_move = 0
    end
  end

  def event_cleared? at_x, at_y
    @cleared[[at_x, at_y]] == true
  end

  def landmark_is_a? x, y, type
    return false if !landmark_at? x, y

    @landmarks[[x,y]] == type
  end

  def distance_to x, y, from_x = @x, from_y = @y
    (from_x - x).abs + (from_y - y).abs
  end

  def distance_to_loc loc
    distance_to loc[:x], loc[:y]
  end

  def move direction, value
    self[[@x, @y]] = :visible
    @previous_x = @x
    @previous_y = @y
    @x = @x + value if direction == :x
    @y = @y + value if direction == :y
    @y = 39 if @y == -1
    @y = 0 if @y == 40
    @x = 89 if @x == -1
    @x = 0 if @x == 90
    self[[@x, @y]] = :you
    commit
    reveal_map
    check_fight
    eat_or_starve
    drink_or_dehydrate
    init_event
  end

  def commit
    if @x == @half_width && @y == @half_height
      swamp = landmark_locations(:swamp)

      if @cleared_landmarks_this_session.include?(swamp)
        @game.add_perk :gastronome
      end

      @revealed_squares_this_session = []
      @cleared_landmarks_this_session = []

      unlock_mines

      process_perks

      @current_embark_ticks = (@embark_after - 5)
    end
  end

  def unlock_mines
    [
      :iron_mine_cleared,
      :coal_mine_cleared,
      :sulphur_mine_cleared
    ].each do |mine_cleared|
      mine = nil
      if mine_cleared == :iron_mine_cleared
        mine = :iron_mine
      elsif mine_cleared == :coal_mine_cleared
        mine = :coal_mine
      elsif mine_cleared == :sulphur_mine_cleared
        mine = :sulphur_mine
      end
      mine_location = landmark_locations(mine)

      if @landmarks[mine_location] == mine_cleared and !@game.buildings[mine] and @game.buildings[:hut]
        @game.buildings[mine] = 1
        if mine_cleared == :iron_mine_cleared
          @game.stores[:iron] ||= 0
          @game.stores[:iron] += 1
        elsif mine_cleared == :coal_mine_cleared
          mine = :coal_mine
          @game.stores[:coal] ||= 0
          @game.stores[:coal] += 1
        elsif mine_cleared == :sulphur_mine_cleared
          mine = :sulphur_mine
          @game.stores[:sulphur] ||= 0
          @game.stores[:sulphur] += 1
        end
        @game.history << "can now send villagers to the #{mine}, just need to give them the food to do so."
      end
    end
  end

  def move_up
    move :y, -1
  end

  def move_down

    move :y, 1
  end

  def move_left
    move :x, -1
  end

  def move_right
    move :x, 1
  end

  def city_cleared?
    result = false

    @cleared.keys.each do |key|
      result = true if @cleared[key] and @landmarks[key] == :city
    end

    result
  end

  def percentage_embark
    return ((@current_embark_ticks / @embark_after) * 100.0).to_i
  end
end
